@@ -246,7 +246,7 @@ aliases_request_basic_pep_cb (GabbleConnection *self,
                               GError *error)
 {
   GabbleConnectionAliasSource source = GABBLE_CONNECTION_ALIAS_NONE;
-  TpHandle handle = GPOINTER_TO_INT (user_data);
+  TpHandle handle = GPOINTER_TO_UINT (user_data);
 
   aliases_request_cache_pep (self, msg, handle, error);
 
@@ -439,6 +439,7 @@ nick_publish_msg_reply_cb (GabbleConnection *conn,
                            GObject *object,
                            gpointer user_data)
 {
+#ifdef ENABLE_DEBUG
   if (lm_message_get_sub_type (reply_msg) == LM_MESSAGE_SUB_TYPE_ERROR)
     {
       LmMessageNode *error_node;
@@ -447,13 +448,15 @@ nick_publish_msg_reply_cb (GabbleConnection *conn,
 
       if (error_node != NULL)
         {
-          GabbleXmppError error = gabble_xmpp_error_from_node (error_node);
+          GabbleXmppError error = gabble_xmpp_error_from_node (error_node,
+              NULL);
 
           DEBUG ("can't publish nick using PEP: %s: %s",
               gabble_xmpp_error_string (error),
               gabble_xmpp_error_description (error));
         }
     }
+#endif
 
   return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 }
@@ -463,7 +466,7 @@ setaliases_foreach (gpointer key, gpointer value, gpointer user_data)
 {
   struct _i_hate_g_hash_table_foreach *data =
     (struct _i_hate_g_hash_table_foreach *) user_data;
-  TpHandle handle = GPOINTER_TO_INT (key);
+  TpHandle handle = GPOINTER_TO_UINT (key);
   gchar *alias = (gchar *) value;
   GError *error = NULL;
   TpBaseConnection *base = (TpBaseConnection *) data->conn;
@@ -912,7 +915,7 @@ maybe_request_vcard (GabbleConnection *self, TpHandle handle,
             tp_base_connection_get_handles (base, TP_HANDLE_TYPE_CONTACT);
 
           gabble_do_pep_request (self, handle, contact_handles,
-            aliases_request_basic_pep_cb, GINT_TO_POINTER (handle));
+            aliases_request_basic_pep_cb, GUINT_TO_POINTER (handle));
         }
       else
         {
