@@ -23,7 +23,7 @@
 
 #include <glib.h>
 
-#include "gabble-connection.h"
+#include "connection.h"
 
 G_BEGIN_DECLS
 
