@@ -50,6 +50,7 @@
 #include "caps-hash.h"
 #include "conn-aliasing.h"
 #include "conn-avatars.h"
+#include "conn-location.h"
 #include "conn-presence.h"
 #include "conn-olpc.h"
 #include "debug.h"
@@ -101,6 +102,8 @@ G_DEFINE_TYPE_WITH_CODE(GabbleConnection,
       tp_presence_mixin_simple_presence_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CONNECTION_INTERFACE_PRESENCE,
       conn_presence_iface_init);
+    G_IMPLEMENT_INTERFACE (GABBLE_TYPE_SVC_CONNECTION_INTERFACE_LOCATION,
+      location_iface_init);
     G_IMPLEMENT_INTERFACE (GABBLE_TYPE_SVC_OLPC_BUDDY_INFO,
       olpc_buddy_info_iface_init);
     G_IMPLEMENT_INTERFACE (GABBLE_TYPE_SVC_OLPC_ACTIVITY_PROPERTIES,
@@ -131,6 +134,8 @@ enum
     PROP_FALLBACK_CONFERENCE_SERVER,
     PROP_STUN_SERVER,
     PROP_STUN_PORT,
+    PROP_FALLBACK_STUN_SERVER,
+    PROP_FALLBACK_STUN_PORT,
     PROP_IGNORE_SSL_ERRORS,
     PROP_ALIAS,
 
@@ -167,6 +172,9 @@ struct _GabbleConnectionPrivate
   gchar *stun_server;
   guint16 stun_port;
 
+  gchar *fallback_stun_server;
+  guint16 fallback_stun_port;
+
   gchar *fallback_conference_server;
 
   /* authentication properties */
@@ -370,6 +378,12 @@ gabble_connection_get_property (GObject    *object,
     case PROP_STUN_PORT:
       g_value_set_uint (value, priv->stun_port);
       break;
+    case PROP_FALLBACK_STUN_SERVER:
+      g_value_set_string (value, priv->fallback_stun_server);
+      break;
+    case PROP_FALLBACK_STUN_PORT:
+      g_value_set_uint (value, priv->fallback_stun_port);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -449,6 +463,13 @@ gabble_connection_set_property (GObject      *object,
     case PROP_STUN_PORT:
       priv->stun_port = g_value_get_uint (value);
       break;
+    case PROP_FALLBACK_STUN_SERVER:
+      g_free (priv->fallback_stun_server);
+      priv->fallback_stun_server = g_value_dup_string (value);
+      break;
+    case PROP_FALLBACK_STUN_PORT:
+      priv->fallback_stun_port = g_value_get_uint (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -533,12 +554,22 @@ gabble_connection_class_init (GabbleConnectionClass *gabble_connection_class)
         { "GadgetAvailable", NULL, NULL },
         { NULL }
   };
+  static TpDBusPropertiesMixinPropImpl location_props[] = {
+        { "LocationAccessControlTypes", NULL, NULL },
+        { "LocationAccessControl", NULL, NULL },
+        { NULL }
+  };
   static TpDBusPropertiesMixinIfaceImpl prop_interfaces[] = {
         { GABBLE_IFACE_OLPC_GADGET,
           conn_olpc_gadget_properties_getter,
           NULL,
           olpc_gadget_props,
         },
+        { GABBLE_IFACE_CONNECTION_INTERFACE_LOCATION,
+          conn_location_properties_getter,
+          conn_location_properties_setter,
+          location_props,
+        },
         { NULL }
   };
 
@@ -686,6 +717,20 @@ gabble_connection_class_init (GabbleConnectionClass *gabble_connection_class)
           0, G_MAXUINT16, GABBLE_PARAMS_DEFAULT_STUN_PORT,
           G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (object_class, PROP_FALLBACK_STUN_SERVER,
+      g_param_spec_string (
+          "fallback-stun-server", "fallback STUN server",
+          "Fallback STUN server.",
+          GABBLE_PARAMS_DEFAULT_FALLBACK_STUN_SERVER,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (object_class, PROP_FALLBACK_STUN_PORT,
+      g_param_spec_uint (
+          "fallback-stun-port", "fallback STUN port",
+          "Fallback STUN port.",
+          0, G_MAXUINT16, GABBLE_PARAMS_DEFAULT_STUN_PORT,
+          G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   g_object_class_install_property (object_class, PROP_IGNORE_SSL_ERRORS,
       g_param_spec_boolean (
           "ignore-ssl-errors", "Ignore SSL errors",
@@ -2028,7 +2073,8 @@ connection_disco_cb (GabbleDisco *disco,
   if (conn->features & GABBLE_CONNECTION_FEATURES_PEP)
     {
       const gchar *ifaces[] = { GABBLE_IFACE_OLPC_BUDDY_INFO,
-          GABBLE_IFACE_OLPC_ACTIVITY_PROPERTIES, NULL };
+          GABBLE_IFACE_OLPC_ACTIVITY_PROPERTIES,
+          GABBLE_IFACE_CONNECTION_INTERFACE_LOCATION, NULL };
 
       tp_base_connection_add_interfaces ((TpBaseConnection *) conn, ifaces);
     }
@@ -2663,7 +2709,7 @@ gabble_connection_get_contact_capabilities (
 
       gabble_connection_get_handle_contact_capabilities (self, handle, arr);
 
-      g_hash_table_insert (ret, GINT_TO_POINTER (handle), arr);
+      g_hash_table_insert (ret, GUINT_TO_POINTER (handle), arr);
     }
 
   gabble_svc_connection_interface_contact_capabilities_return_from_get_contact_capabilities
