@@ -38,12 +38,12 @@
 #include "connection.h"
 #include "debug.h"
 #include "disco.h"
+#include "message-util.h"
 #include "presence.h"
 #include "presence-cache.h"
 #include "roster.h"
 
 static void channel_iface_init (gpointer, gpointer);
-static void text_iface_init (gpointer, gpointer);
 static void chat_state_iface_init (gpointer, gpointer);
 static void destroyable_iface_init (gpointer, gpointer);
 
@@ -51,7 +51,10 @@ G_DEFINE_TYPE_WITH_CODE (GabbleIMChannel, gabble_im_channel, G_TYPE_OBJECT,
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_DBUS_PROPERTIES,
       tp_dbus_properties_mixin_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL, channel_iface_init);
-    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_TYPE_TEXT, text_iface_init);
+    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_TYPE_TEXT,
+      tp_message_mixin_text_iface_init);
+    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_MESSAGES,
+      tp_message_mixin_messages_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_IFACE, NULL);
     G_IMPLEMENT_INTERFACE (TP_TYPE_EXPORTABLE_CHANNEL, NULL);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_CHAT_STATE,
@@ -59,8 +62,12 @@ G_DEFINE_TYPE_WITH_CODE (GabbleIMChannel, gabble_im_channel, G_TYPE_OBJECT,
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_DESTROYABLE,
       destroyable_iface_init));
 
+static void _gabble_im_channel_send_message (GObject *object,
+    TpMessage *message, TpMessageSendingFlags flags);
+
 static const gchar *gabble_im_channel_interfaces[] = {
     TP_IFACE_CHANNEL_INTERFACE_CHAT_STATE,
+    TP_IFACE_CHANNEL_INTERFACE_MESSAGES,
     TP_IFACE_CHANNEL_INTERFACE_DESTROYABLE,
     NULL
 };
@@ -94,6 +101,7 @@ struct _GabbleIMChannelPrivate
   TpHandle initiator;
 
   gchar *peer_jid;
+  gboolean send_nick;
 
   gboolean closed;
   gboolean dispose_has_run;
@@ -110,6 +118,8 @@ gabble_im_channel_init (GabbleIMChannel *self)
   self->priv = priv;
 }
 
+#define NUM_SUPPORTED_MESSAGE_TYPES 3
+
 static GObject *
 gabble_im_channel_constructor (GType type, guint n_props,
                                GObjectConstructParam *props)
@@ -118,9 +128,19 @@ gabble_im_channel_constructor (GType type, guint n_props,
   GabbleIMChannelPrivate *priv;
   TpBaseConnection *conn;
   DBusGConnection *bus;
-  gboolean send_nick;
   TpHandleRepoIface *contact_handles;
 
+  TpChannelTextMessageType types[NUM_SUPPORTED_MESSAGE_TYPES] = {
+      TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL,
+      TP_CHANNEL_TEXT_MESSAGE_TYPE_ACTION,
+      TP_CHANNEL_TEXT_MESSAGE_TYPE_NOTICE,
+  };
+
+  const gchar * supported_content_types[] = {
+      "text/plain",
+      NULL
+  };
+
   obj = G_OBJECT_CLASS (gabble_im_channel_parent_class)->
            constructor (type, n_props, props);
   priv = GABBLE_IM_CHANNEL_GET_PRIVATE (GABBLE_IM_CHANNEL (obj));
@@ -141,18 +161,17 @@ gabble_im_channel_constructor (GType type, guint n_props,
 
   if (gabble_roster_handle_get_subscription (priv->conn->roster, priv->handle)
         & GABBLE_ROSTER_SUBSCRIPTION_FROM)
-    send_nick = FALSE;
+    priv->send_nick = FALSE;
   else
-    send_nick = TRUE;
+    priv->send_nick = TRUE;
 
-  gabble_text_mixin_init (obj, G_STRUCT_OFFSET (GabbleIMChannel, text),
-      contact_handles, send_nick);
+  tp_message_mixin_init (obj, G_STRUCT_OFFSET (GabbleIMChannel, message_mixin),
+      conn);
 
-  tp_text_mixin_set_message_types (obj,
-      TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL,
-      TP_CHANNEL_TEXT_MESSAGE_TYPE_ACTION,
-      TP_CHANNEL_TEXT_MESSAGE_TYPE_NOTICE,
-      G_MAXUINT);
+  tp_message_mixin_implement_sending (obj, _gabble_im_channel_send_message,
+      NUM_SUPPORTED_MESSAGE_TYPES, types, 0,
+      TP_DELIVERY_REPORTING_SUPPORT_FLAG_RECEIVE_FAILURES,
+      supported_content_types);
 
   return obj;
 }
@@ -361,12 +380,11 @@ gabble_im_channel_class_init (GabbleIMChannelClass *gabble_im_channel_class)
   g_object_class_install_property (object_class, PROP_INITIATOR_ID,
       param_spec);
 
-  tp_text_mixin_class_init (object_class,
-      G_STRUCT_OFFSET (GabbleIMChannelClass, text_class));
-
   gabble_im_channel_class->dbus_props_class.interfaces = prop_interfaces;
   tp_dbus_properties_mixin_class_init (object_class,
       G_STRUCT_OFFSET (GabbleIMChannelClass, dbus_props_class));
+
+  tp_message_mixin_init_dbus_properties (object_class);
 }
 
 static void
@@ -410,10 +428,8 @@ gabble_im_channel_dispose (GObject *object)
           {
           /* Set the chat state of the channel on gone
            * (Channel.Interface.ChatState) */
-          gabble_text_mixin_send (G_OBJECT (self),
-              TP_CHANNEL_TEXT_MESSAGE_TYPE_NOTICE, 0,
-              TP_CHANNEL_CHAT_STATE_GONE, priv->peer_jid, NULL, priv->conn,
-              FALSE /* emit_signal */, NULL);
+          gabble_message_util_send_chat_state (G_OBJECT (self), priv->conn,
+              0, TP_CHANNEL_CHAT_STATE_GONE, priv->peer_jid, NULL);
           }
 
         tp_svc_channel_emit_closed (self);
@@ -444,11 +460,44 @@ gabble_im_channel_finalize (GObject *object)
   g_free (priv->object_path);
   g_free (priv->peer_jid);
 
-  tp_text_mixin_finalize (object);
+  tp_message_mixin_finalize (object);
 
   G_OBJECT_CLASS (gabble_im_channel_parent_class)->finalize (object);
 }
 
+
+static void
+_gabble_im_channel_send_message (GObject *object,
+                                 TpMessage *message,
+                                 TpMessageSendingFlags flags)
+{
+  GabbleIMChannel *self = GABBLE_IM_CHANNEL (object);
+  GabbleIMChannelPrivate *priv;
+  GabblePresence *presence;
+  gint state = -1;
+
+  g_assert (GABBLE_IS_IM_CHANNEL (self));
+  priv = GABBLE_IM_CHANNEL_GET_PRIVATE (self);
+
+  presence = gabble_presence_cache_get (priv->conn->presence_cache,
+      priv->handle);
+
+  if (presence && (presence->caps & PRESENCE_CAP_CHAT_STATES))
+    {
+      state = TP_CHANNEL_CHAT_STATE_ACTIVE;
+    }
+
+  /* We don't support providing successful delivery reports. */
+  flags = 0;
+
+  gabble_message_util_send_message (object, priv->conn, message, flags, 0,
+      state, priv->peer_jid, priv->send_nick);
+
+  if (priv->send_nick)
+    priv->send_nick = FALSE;
+}
+
+
 /**
  * _gabble_im_channel_receive
  *
@@ -459,24 +508,70 @@ _gabble_im_channel_receive (GabbleIMChannel *chan,
                             TpHandle sender,
                             const char *from,
                             time_t timestamp,
-                            const char *text)
+                            const char *text,
+                            TpChannelTextSendError send_error,
+                            TpDeliveryStatus delivery_status)
 {
   GabbleIMChannelPrivate *priv;
+  TpBaseConnection *base_conn;
+  TpMessage *msg;
 
   g_assert (GABBLE_IS_IM_CHANNEL (chan));
   priv = GABBLE_IM_CHANNEL_GET_PRIVATE (chan);
+  base_conn = (TpBaseConnection *) priv->conn;
 
   /* update peer's full JID if it's changed */
-  if (0 != strcmp (from, priv->peer_jid))
+  if (send_error != GABBLE_TEXT_CHANNEL_SEND_NO_ERROR &&
+      0 != strcmp (from, priv->peer_jid))
     {
       g_free (priv->peer_jid);
       priv->peer_jid = g_strdup (from);
     }
 
-  if (timestamp == 0)
-      timestamp = time (NULL);
+  msg = tp_message_new (base_conn, 2, 2);
 
-  tp_text_mixin_receive (G_OBJECT (chan), type, sender, timestamp, text);
+  /* Header */
+  if (type != TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL)
+    tp_message_set_uint32 (msg, 0, "message-type", type);
+
+  if (timestamp != 0)
+    tp_message_set_uint64 (msg, 0, "message-sent", timestamp);
+
+  /* Body */
+  tp_message_set_string (msg, 1, "content-type", "text/plain");
+  tp_message_set_string (msg, 1, "content", text);
+
+  if (send_error == GABBLE_TEXT_CHANNEL_SEND_NO_ERROR)
+    {
+      tp_message_set_handle (msg, 0, "message-sender", TP_HANDLE_TYPE_CONTACT,
+          sender);
+      tp_message_set_uint64 (msg, 0, "message-received", time (NULL));
+
+      tp_message_mixin_take_received (G_OBJECT (chan), msg);
+    }
+  else
+    {
+      TpMessage *delivery_report = tp_message_new (base_conn, 1, 1);
+
+      tp_message_set_uint32 (delivery_report, 0, "message-type",
+          TP_CHANNEL_TEXT_MESSAGE_TYPE_DELIVERY_REPORT);
+      tp_message_set_handle (delivery_report, 0, "message-sender",
+          TP_HANDLE_TYPE_CONTACT, sender);
+      tp_message_set_uint64 (delivery_report, 0, "message-received",
+          time (NULL));
+
+      tp_message_set_uint32 (delivery_report, 0, "delivery-status",
+          delivery_status);
+      tp_message_set_uint32 (delivery_report, 0, "delivery-error", send_error);
+
+      /* We're getting a send error, so the original sender of the echoed
+       * message must be us! */
+      tp_message_set_handle (msg, 0, "message-sender", TP_HANDLE_TYPE_CONTACT,
+          base_conn->self_handle);
+      tp_message_take_message (delivery_report, 0, "delivery-echo", msg);
+
+      tp_message_mixin_take_received (G_OBJECT (chan), delivery_report);
+    }
 }
 
 /**
@@ -533,7 +628,7 @@ gabble_im_channel_close (TpSvcChannel *iface,
       /* The IM factory will resurrect the channel if we have pending
        * messages. When we're resurrected, we want the initiator
        * to be the contact who sent us those messages, if it isn't already */
-      if (tp_text_mixin_has_pending_messages ((GObject *) self, NULL))
+      if (tp_message_mixin_has_pending_messages ((GObject *) self, NULL))
         {
           DEBUG ("Not really closing, I still have pending messages");
 
@@ -550,7 +645,7 @@ gabble_im_channel_close (TpSvcChannel *iface,
               tp_handle_ref (contact_repo, priv->initiator);
             }
 
-          tp_text_mixin_set_rescued ((GObject *) self);
+          tp_message_mixin_set_rescued ((GObject *) self);
         }
       else
         {
@@ -563,12 +658,10 @@ gabble_im_channel_close (TpSvcChannel *iface,
 
       if (presence && (presence->caps & PRESENCE_CAP_CHAT_STATES))
         {
-          /* Set the chat state of the channel on gone
+          /* Set the chat state of the channel to gone
            * (Channel.Interface.ChatState) */
-          gabble_text_mixin_send (G_OBJECT (self),
-              TP_CHANNEL_TEXT_MESSAGE_TYPE_NOTICE, 0,
-              TP_CHANNEL_CHAT_STATE_GONE, priv->peer_jid, NULL, priv->conn,
-              FALSE /* emit_signal */, NULL);
+          gabble_message_util_send_chat_state (G_OBJECT (self), priv->conn,
+              0, TP_CHANNEL_CHAT_STATE_GONE, priv->peer_jid, NULL);
         }
     }
 
@@ -607,7 +700,7 @@ gabble_im_channel_destroy (TpSvcChannelInterfaceDestroyable *iface,
   DEBUG ("called on %p", self);
 
   /* Clear out any pending messages */
-  tp_text_mixin_clear ((GObject *) self);
+  tp_message_mixin_clear ((GObject *) self);
 
   /* Close() and Destroy() have the same signature, so we can safely
    * chain to the other function now */
@@ -651,48 +744,6 @@ gabble_im_channel_get_interfaces (TpSvcChannel *iface,
 }
 
 
-/**
- * gabble_im_channel_send
- *
- * Implements D-Bus method Send
- * on interface org.freedesktop.Telepathy.Channel.Type.Text
- */
-static void
-gabble_im_channel_send (TpSvcChannelTypeText *iface,
-                        guint type,
-                        const gchar *text,
-                        DBusGMethodInvocation *context)
-{
-  GabbleIMChannel *self = GABBLE_IM_CHANNEL (iface);
-  GabbleIMChannelPrivate *priv;
-  GabblePresence *presence;
-  gint state = -1;
-  GError *error = NULL;
-
-  g_assert (GABBLE_IS_IM_CHANNEL (self));
-  priv = GABBLE_IM_CHANNEL_GET_PRIVATE (self);
-
-  presence = gabble_presence_cache_get (priv->conn->presence_cache,
-      priv->handle);
-
-  if (presence && (presence->caps & PRESENCE_CAP_CHAT_STATES))
-    {
-      state = TP_CHANNEL_CHAT_STATE_ACTIVE;
-    }
-
-  if (!gabble_text_mixin_send (G_OBJECT (self), type, 0,
-      state, priv->peer_jid, text, priv->conn,
-      TRUE /* emit_signal */, &error))
-    {
-      dbus_g_method_return_error (context, error);
-      g_error_free (error);
-
-      return;
-    }
-
-  tp_svc_channel_type_text_return_from_send (context);
-}
-
 /**
  * gabble_im_channel_set_chat_state
  *
@@ -734,9 +785,9 @@ gabble_im_channel_set_chat_state (TpSvcChannelInterfaceChatState *iface,
               "you may not explicitly set the Gone state");
         }
 
-      if (error != NULL || !gabble_text_mixin_send (G_OBJECT (self),
-          TP_CHANNEL_TEXT_MESSAGE_TYPE_NOTICE, 0, state, priv->peer_jid, NULL,
-          priv->conn, FALSE /* emit_signal */, &error))
+      if (error != NULL ||
+          !gabble_message_util_send_chat_state (G_OBJECT (self), priv->conn,
+              0, state, priv->peer_jid, &error))
         {
           dbus_g_method_return_error (context, error);
           g_error_free (error);
@@ -766,18 +817,6 @@ channel_iface_init (gpointer g_iface, gpointer iface_data)
 #undef IMPLEMENT
 }
 
-static void
-text_iface_init (gpointer g_iface, gpointer iface_data)
-{
-  TpSvcChannelTypeTextClass *klass = (TpSvcChannelTypeTextClass *) g_iface;
-
-  tp_text_mixin_iface_init (g_iface, iface_data);
-#define IMPLEMENT(x) tp_svc_channel_type_text_implement_##x (\
-    klass, gabble_im_channel_##x)
-  IMPLEMENT(send);
-#undef IMPLEMENT
-}
-
 static void
 chat_state_iface_init (gpointer g_iface, gpointer iface_data)
 {
