@@ -58,6 +58,7 @@ enum
   PROP_SENDERS,
   PROP_STATE,
   PROP_DISPOSITION,
+  PROP_LOCALLY_CREATED,
   LAST_PROPERTY
 };
 
@@ -65,7 +66,6 @@ struct _GabbleJingleContentPrivate
 {
   gchar *name;
   gchar *creator;
-  gboolean created_by_initiator;
   gboolean created_by_us;
   JingleContentState state;
   JingleContentSenders senders;
@@ -80,6 +80,7 @@ struct _GabbleJingleContentPrivate
   gboolean transport_ready;
 
   guint timer_id;
+  guint gtalk4_event_id;
 
   gboolean dispose_has_run;
 };
@@ -104,11 +105,11 @@ gabble_jingle_content_init (GabbleJingleContent *obj)
   obj->priv = priv;
 
   priv->state = JINGLE_CONTENT_STATE_EMPTY;
-  priv->created_by_initiator = TRUE;
   priv->created_by_us = TRUE;
   priv->media_ready = FALSE;
   priv->transport_ready = FALSE;
   priv->timer_id = 0;
+  priv->gtalk4_event_id = 0;
   priv->dispose_has_run = FALSE;
 
   obj->conn = NULL;
@@ -127,7 +128,20 @@ gabble_jingle_content_dispose (GObject *object)
   DEBUG ("dispose called");
   priv->dispose_has_run = TRUE;
 
-  g_assert (priv->timer_id == 0);
+  /* If we're in the middle of content-add/-accept when the session is
+   * terminated, we'll get disposed without being explicitly removed from
+   * the session. So, remove the timer here. */
+  if (priv->timer_id != 0)
+    {
+      g_source_remove (priv->timer_id);
+      priv->timer_id = 0;
+    }
+
+  if (priv->gtalk4_event_id != 0)
+    {
+      g_source_remove (priv->gtalk4_event_id);
+      priv->gtalk4_event_id = 0;
+    }
 
   g_free (priv->name);
   priv->name = NULL;
@@ -182,6 +196,9 @@ gabble_jingle_content_get_property (GObject *object,
     case PROP_DISPOSITION:
       g_value_set_string (value, priv->disposition);
       break;
+    case PROP_LOCALLY_CREATED:
+      g_value_set_boolean (value, priv->created_by_us);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -217,14 +234,13 @@ gabble_jingle_content_set_property (GObject *object,
 
       if (priv->transport_ns != NULL)
         {
-          GType transport_type = GPOINTER_TO_INT (
-              g_hash_table_lookup (self->conn->jingle_factory->transports,
-                  priv->transport_ns));
+          GType transport_type = gabble_jingle_factory_lookup_transport (
+              self->conn->jingle_factory, priv->transport_ns);
 
           g_assert (transport_type != 0);
 
-          priv->transport = g_object_new (transport_type,
-              "content", self, "transport-ns", priv->transport_ns, NULL);
+          priv->transport = gabble_jingle_transport_iface_new (transport_type,
+              self, priv->transport_ns);
 
           g_signal_connect (priv->transport, "new-candidates",
               (GCallback) new_transport_candidates_cb, self);
@@ -336,6 +352,14 @@ gabble_jingle_content_class_init (GabbleJingleContentClass *cls)
                                     G_PARAM_STATIC_BLURB);
   g_object_class_install_property (object_class, PROP_DISPOSITION, param_spec);
 
+  param_spec = g_param_spec_boolean ("locally-created", "Locally created",
+                                     "True if the content was created by the "
+                                     "local client.",
+                                     FALSE,
+                                     G_PARAM_READABLE |
+                                     G_PARAM_STATIC_NAME |
+                                     G_PARAM_STATIC_BLURB);
+  g_object_class_install_property (object_class, PROP_LOCALLY_CREATED, param_spec);
 
   /* signal definitions */
 
@@ -480,8 +504,8 @@ gabble_jingle_content_parse_add (GabbleJingleContent *c,
 
           dialect = JINGLE_DIALECT_GTALK3;
           g_object_set (c->session, "dialect", JINGLE_DIALECT_GTALK3, NULL);
-          transport_type = GPOINTER_TO_INT (
-              g_hash_table_lookup (c->conn->jingle_factory->transports, ""));
+          transport_type = gabble_jingle_factory_lookup_transport (
+              c->conn->jingle_factory, "");
           priv->transport_ns = g_strdup ("");
         }
     }
@@ -499,8 +523,8 @@ gabble_jingle_content_parse_add (GabbleJingleContent *c,
     {
       const gchar *ns = lm_message_node_get_namespace (trans_node);
 
-      transport_type = GPOINTER_TO_INT (
-          g_hash_table_lookup (c->conn->jingle_factory->transports, ns));
+      transport_type = gabble_jingle_factory_lookup_transport (
+          c->conn->jingle_factory, ns);
 
       if (transport_type == 0)
         {
@@ -512,7 +536,6 @@ gabble_jingle_content_parse_add (GabbleJingleContent *c,
     }
 
   priv->created_by_us = FALSE;
-  priv->created_by_initiator = (!tp_strdiff (creator, "initiator"));
   priv->senders = parse_senders (senders);
   if (priv->senders == JINGLE_CONTENT_SENDERS_NONE)
     {
@@ -532,10 +555,8 @@ gabble_jingle_content_parse_add (GabbleJingleContent *c,
 
   DEBUG ("content creating new transport type %s", g_type_name (transport_type));
 
-  trans = g_object_new (transport_type,
-                       "content", c,
-                       "transport-ns", priv->transport_ns,
-                       NULL);
+  trans = gabble_jingle_transport_iface_new (transport_type,
+      c, priv->transport_ns);
 
   g_signal_connect (trans, "new-candidates",
       (GCallback) new_transport_candidates_cb, c);
@@ -566,7 +587,7 @@ gabble_jingle_content_parse_add (GabbleJingleContent *c,
    * the transport type? */
   if (dialect == JINGLE_DIALECT_GTALK4)
     {
-      g_idle_add (send_gtalk4_transport_accept, c);
+      priv->gtalk4_event_id = g_idle_add (send_gtalk4_transport_accept, c);
     }
 
   return;
@@ -620,6 +641,10 @@ gabble_jingle_content_parse_accept (GabbleJingleContent *c,
   if (*error != NULL)
       return;
 
+  if (priv->timer_id != 0)
+      g_source_remove (priv->timer_id);
+  priv->timer_id = 0;
+
   priv->state = JINGLE_CONTENT_STATE_ACKNOWLEDGED;
   g_object_notify ((GObject *) c, "state");
 }
@@ -647,12 +672,21 @@ gabble_jingle_content_produce_node (GabbleJingleContent *c,
     }
   else
     {
+      gboolean session_created_by_us;
+
       content_node = lm_message_node_add_child (parent, "content", NULL);
       lm_message_node_set_attributes (content_node,
-          "creator", priv->created_by_initiator ? "initiator" : "responder",
           "name", priv->name,
           "senders", produce_senders (priv->senders),
           NULL);
+
+      g_object_get (c->session, "local-initiator", &session_created_by_us,
+        NULL);
+
+      if (priv->created_by_us == session_created_by_us)
+        lm_message_node_set_attribute (content_node, "creator", "initiator");
+      else
+        lm_message_node_set_attribute (content_node, "creator", "responder");
     }
 
   if (!full)
