@@ -78,6 +78,7 @@ struct _GabbleJingleSessionPrivate
   gchar *sid;
 
   gboolean locally_accepted;
+  gboolean locally_terminated;
 
   guint timer_id;
 
@@ -122,8 +123,6 @@ static JingleAction allowed_actions[MAX_JINGLE_STATES][MAX_ACTIONS_PER_STATE] =
   { JINGLE_ACTION_UNKNOWN }
 };
 
-static gboolean _terminate_delayed (gpointer user_data);
-
 static void
 gabble_jingle_session_init (GabbleJingleSession *obj)
 {
@@ -137,6 +136,7 @@ gabble_jingle_session_init (GabbleJingleSession *obj)
 
   priv->state = JS_STATE_PENDING_CREATED;
   priv->locally_accepted = FALSE;
+  priv->locally_terminated = FALSE;
   priv->timer_id = 0;
   priv->dispose_has_run = FALSE;
 }
@@ -266,6 +266,22 @@ gabble_jingle_session_set_property (GObject *object,
   }
 }
 
+GabbleJingleSession *
+gabble_jingle_session_new (GabbleConnection *connection,
+                           const gchar *session_id,
+                           gboolean local_initiator,
+                           TpHandle peer,
+                           const gchar *peer_resource)
+{
+  return g_object_new (GABBLE_TYPE_JINGLE_SESSION,
+      "session-id", session_id,
+      "connection", connection,
+      "local-initiator", local_initiator,
+      "peer", peer,
+      "peer-resource", peer_resource,
+      NULL);
+}
+
 static void
 gabble_jingle_session_class_init (GabbleJingleSessionClass *cls)
 {
@@ -352,7 +368,7 @@ gabble_jingle_session_class_init (GabbleJingleSessionClass *cls)
   signals[NEW_CONTENT] = g_signal_new ("new-content",
         G_TYPE_FROM_CLASS (cls), G_SIGNAL_RUN_LAST,
         0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT,
-        G_TYPE_NONE, 1, G_TYPE_POINTER);
+        G_TYPE_NONE, 1, G_TYPE_OBJECT);
 
   signals[TERMINATED] = g_signal_new ("terminated",
         G_TYPE_FROM_CLASS (cls), G_SIGNAL_RUN_LAST,
@@ -540,6 +556,7 @@ fire_idle_content_reject (GabbleJingleSession *sess, const gchar *name,
   ctx->name = g_strdup (name);
   ctx->creator = g_strdup (creator);
 
+  /* FIXME: add API for ordering IQs rather than using g_idle_add. */
   g_idle_add (idle_content_reject, ctx);
 }
 
@@ -552,6 +569,9 @@ create_content (GabbleJingleSession *sess, GType content_type,
   GabbleJingleContent *c;
 
   DEBUG ("session creating new content type, conn == %p, jf == %p", priv->conn, priv->conn->jingle_factory);
+
+  /* FIXME: media-type is introduced by GabbleJingleMediaRTP, not by the
+   * superclass, so this call is unsafe in the general case */
   c = g_object_new (content_type,
                     "connection", priv->conn,
                     "session", sess,
@@ -607,9 +627,8 @@ _each_content_add (GabbleJingleSession *sess, GabbleJingleContent *c,
     {
       content_ns = lm_message_node_get_namespace (desc_node);
       DEBUG ("namespace: %s", content_ns);
-      content_type =
-          GPOINTER_TO_INT (g_hash_table_lookup (priv->conn->jingle_factory->content_types,
-          content_ns));
+      content_type = gabble_jingle_factory_lookup_content_type (
+          priv->conn->jingle_factory, content_ns);
     }
 
   if (content_type == 0)
@@ -715,6 +734,7 @@ _each_content_accept (GabbleJingleSession *sess, GabbleJingleContent *c,
     LmMessageNode *content_node ,GError **error)
 {
   GabbleJingleSessionPrivate *priv = GABBLE_JINGLE_SESSION_GET_PRIVATE (sess);
+  JingleContentState state;
 
   if (c == NULL)
     {
@@ -723,6 +743,16 @@ _each_content_accept (GabbleJingleSession *sess, GabbleJingleContent *c,
       return;
     }
 
+  g_object_get (c, "state", &state, NULL);
+  if (state != JINGLE_CONTENT_STATE_SENT)
+    {
+#ifdef ENABLE_DEBUG
+      const gchar *name = lm_message_node_get_attribute (content_node, "name");
+      DEBUG ("ignoring content \"%s\"s acceptance for content not in SENT state", name);
+#endif
+      return;
+    }
+
   gabble_jingle_content_parse_accept (c, content_node,
       JINGLE_IS_GOOGLE_DIALECT (priv->dialect), error);
 }
@@ -738,7 +768,7 @@ on_session_initiate (GabbleJingleSession *sess, LmMessageNode *node,
     {
       /* We ignore initiate from us, and terminate the session immediately
        * afterwards */
-      _terminate_delayed (sess);
+      gabble_jingle_session_terminate (sess);
       return;
     }
 
@@ -1167,7 +1197,9 @@ gabble_jingle_session_new_message (GabbleJingleSession *sess,
   gchar *el = NULL, *ns = NULL;
   gboolean gtalk_mode = FALSE;
 
-  g_assert ((action == JINGLE_ACTION_SESSION_INITIATE) || (priv->state > JS_STATE_PENDING_CREATED));
+  g_assert ((action == JINGLE_ACTION_SESSION_INITIATE) ||
+            (priv->state > JS_STATE_PENDING_CREATED));
+  g_assert (GABBLE_IS_JINGLE_SESSION (sess));
 
   /* possibly this is the first message in an outgoing session,
    * meaning that we have to set up initiator */
@@ -1468,7 +1500,7 @@ set_state (GabbleJingleSession *sess, JingleState state)
     }
 
   if (state == JS_STATE_ENDED)
-      g_signal_emit (sess, signals[TERMINATED], 0, FALSE);
+      g_signal_emit (sess, signals[TERMINATED], 0, priv->locally_terminated);
 }
 
 void
@@ -1504,17 +1536,10 @@ gabble_jingle_session_terminate (GabbleJingleSession *sess)
    * takes care of cleanup */
 
   DEBUG ("we are terminating this session");
+  priv->locally_terminated = TRUE;
   set_state (sess, JS_STATE_ENDED);
 }
 
-static gboolean
-_terminate_delayed (gpointer user_data)
-{
-  GabbleJingleSession *sess = user_data;
-  gabble_jingle_session_terminate (sess);
-  return FALSE;
-}
-
 static void
 _foreach_count_active_contents (gpointer key, gpointer value, gpointer user_data)
 {
@@ -1554,12 +1579,7 @@ content_removed_cb (GabbleJingleContent *c, gpointer user_data)
       return;
 
   if (count_active_contents (sess) == 0)
-    {
-      /* Terminate the session from idle loop
-       * so that content removal can be processed
-       * in media stream before that. */
-      g_idle_add (_terminate_delayed, sess);
-    }
+    gabble_jingle_session_terminate (sess);
 }
 
 
@@ -1596,9 +1616,8 @@ gabble_jingle_session_add_content (GabbleJingleSession *sess, JingleMediaType mt
     }
   while (g_hash_table_lookup (priv->contents, name) != NULL);
 
-  content_type =
-      GPOINTER_TO_INT (g_hash_table_lookup (priv->conn->jingle_factory->content_types,
-      content_ns));
+  content_type = gabble_jingle_factory_lookup_content_type (
+      priv->conn->jingle_factory, content_ns);
 
   g_assert (content_type != 0);
 
