@@ -46,6 +46,7 @@
 #include "namespaces.h"
 #include "presence-cache.h"
 #include "presence.h"
+#include "util.h"
 
 #define MAX_STREAMS 99
 
@@ -195,12 +196,12 @@ static void session_terminated_cb (GabbleJingleSession *session,
     gboolean local_terminator, gpointer user_data);
 static void session_new_content_cb (GabbleJingleSession *session,
     GabbleJingleContent *c, gpointer user_data);
-static GabbleMediaStream *
-create_stream_from_content (GabbleMediaChannel *chan, GabbleJingleContent *c);
+static void create_stream_from_content (GabbleMediaChannel *chan,
+    GabbleJingleContent *c);
 static gboolean contact_is_media_capable (GabbleMediaChannel *chan, TpHandle peer,
     gboolean *wait);
 
-static gboolean
+static void
 _create_streams (GabbleMediaChannel *chan)
 {
   GabbleMediaChannelPrivate *priv = GABBLE_MEDIA_CHANNEL_GET_PRIVATE (chan);
@@ -213,8 +214,6 @@ _create_streams (GabbleMediaChannel *chan)
     }
 
   g_list_free (contents);
-
-  return FALSE;
 }
 
 static void
@@ -270,6 +269,9 @@ gabble_media_channel_constructor (GType type, guint n_props,
   TpIntSet *set;
   TpHandleRepoIface *contact_handles;
   GabbleJingleFactory *jf;
+  const gchar *relay_token;
+  gchar *stun_server;
+  guint stun_port;
 
   obj = G_OBJECT_CLASS (gabble_media_channel_parent_class)->
            constructor (type, n_props, props);
@@ -309,12 +311,25 @@ gabble_media_channel_constructor (GType type, guint n_props,
 
   /* Set up Google relay related properties */
   jf = priv->conn->jingle_factory;
-  if (jf->stun_server != NULL)
-      g_object_set (obj, "stun-server", jf->stun_server, NULL);
-  if (jf->stun_port != 0)
-      g_object_set (obj, "stun-port", jf->stun_port, NULL);
-  if (jf->relay_token != NULL)
-      g_object_set (obj, "gtalk-p2p-relay-token", jf->relay_token, NULL);
+
+  if (gabble_jingle_factory_get_stun_server (jf, &stun_server,
+        &stun_port))
+    {
+      g_object_set (obj,
+          "stun-server", stun_server,
+          "stun-port", stun_port,
+          NULL);
+      g_free (stun_server);
+    }
+
+  relay_token = gabble_jingle_factory_get_google_relay_token (jf);
+
+  if (relay_token != NULL)
+    {
+      g_object_set (obj,
+          "gtalk-p2p-relay-token", relay_token,
+          NULL);
+    }
 
   /* act on incoming session */
   if (priv->session != NULL)
@@ -332,11 +347,9 @@ gabble_media_channel_constructor (GType type, guint n_props,
       tp_group_mixin_change_flags (obj,
           TP_CHANNEL_GROUP_FLAG_CAN_ADD | TP_CHANNEL_GROUP_FLAG_CAN_REMOVE, 0);
 
-      /* We want streams to appear on DBus after the channel is signalled */
-      g_idle_add ((GSourceFunc) _create_streams, GABBLE_MEDIA_CHANNEL (obj));
-
       /* Set up signal callbacks, emit session handler, initialize streams */
       _latch_to_session (GABBLE_MEDIA_CHANNEL (obj));
+      _create_streams (GABBLE_MEDIA_CHANNEL (obj));
     }
 
   return obj;
@@ -505,6 +518,15 @@ gabble_media_channel_set_property (GObject     *object,
       break;
     case PROP_INITIAL_PEER:
       priv->initial_peer = g_value_get_uint (value);
+
+      if (priv->initial_peer != 0)
+        {
+          TpBaseConnection *base_conn = (TpBaseConnection *) priv->conn;
+          TpHandleRepoIface *repo = tp_base_connection_get_handles (base_conn,
+              TP_HANDLE_TYPE_CONTACT);
+          tp_handle_ref (repo, priv->initial_peer);
+        }
+
       break;
     case PROP_SESSION:
       g_assert (priv->session == NULL);
@@ -721,6 +743,12 @@ gabble_media_channel_dispose (GObject *object)
   tp_handle_unref (contact_handles, priv->creator);
   priv->creator = 0;
 
+  if (priv->initial_peer != 0)
+    {
+      tp_handle_unref (contact_handles, priv->initial_peer);
+      priv->initial_peer = 0;
+    }
+
   /** In this we set the state to ENDED, then the callback unrefs
    * the session
    */
@@ -1161,6 +1189,12 @@ _pick_best_content_type (GabbleMediaChannel *chan, TpHandle peer,
 
   presence = gabble_presence_cache_get (priv->conn->presence_cache, peer);
 
+  if (presence == NULL)
+    {
+      DEBUG ("contact %d has no presence available", peer);
+      return NULL;
+    }
+
   if (gabble_presence_resource_has_caps (presence, resource,
           PRESENCE_CAP_JINGLE_RTP))
     {
@@ -1203,6 +1237,12 @@ _pick_best_resource (GabbleMediaChannel *chan,
 
   presence = gabble_presence_cache_get (priv->conn->presence_cache, peer);
 
+  if (presence == NULL)
+    {
+      DEBUG ("contact %d has no presence available", peer);
+      return NULL;
+    }
+
   *dialect = JINGLE_DIALECT_ERROR;
   *transport_ns = NULL;
 
@@ -1797,6 +1837,10 @@ _gabble_media_channel_add_member (GObject *obj,
           tp_group_mixin_change_flags (obj,
               0, TP_CHANNEL_GROUP_FLAG_CAN_ADD);
 
+          /* accept any local pending sends */
+          g_ptr_array_foreach (priv->streams,
+              (GFunc) gabble_media_stream_accept_pending_local_send, NULL);
+
           /* signal acceptance */
           gabble_jingle_session_accept (priv->session);
 
@@ -2283,23 +2327,28 @@ _gabble_media_channel_typeflags_to_caps (TpChannelMediaCapabilities flags)
   return caps;
 }
 
-static GabbleMediaStream *
-create_stream_from_content (GabbleMediaChannel *chan, GabbleJingleContent *c)
+static void
+create_stream_from_content (GabbleMediaChannel *chan,
+                            GabbleJingleContent *c)
 {
+  GObject *chan_o = (GObject *) chan;
   GabbleMediaChannelPrivate *priv = GABBLE_MEDIA_CHANNEL_GET_PRIVATE (chan);
   GabbleMediaStream *stream;
-  JingleMediaType type;
   TpMediaStreamType mtype;
   gchar *name;
   guint id;
   gchar *object_path;
+  gchar *nat_traversal;
 
-  g_object_get (c, "name", &name, "media-type", &type, NULL);
+  g_object_get (c,
+      "name", &name,
+      NULL);
 
   if (G_OBJECT_TYPE (c) != GABBLE_TYPE_JINGLE_MEDIA_RTP)
     {
       DEBUG ("ignoring non MediaRtp content '%s'", name);
-      return NULL;
+      g_free (name);
+      return;
     }
 
   /* This onelier replaces "get_channel_stream_id()" function */
@@ -2309,38 +2358,35 @@ create_stream_from_content (GabbleMediaChannel *chan, GabbleJingleContent *c)
   object_path = g_strdup_printf ("%s/MediaStream%u",
       priv->object_path, id);
 
-  mtype = (type == JINGLE_MEDIA_TYPE_AUDIO) ?
-    TP_MEDIA_STREAM_TYPE_AUDIO : TP_MEDIA_STREAM_TYPE_VIDEO;
-
-  stream = g_object_new (GABBLE_TYPE_MEDIA_STREAM,
-      "object-path", object_path,
-      "content", c,
-      "name", name,
-      "id", id,
-      "media-type", mtype,
+  g_object_get (chan,
+      "nat-traversal", &nat_traversal,
       NULL);
+  stream = gabble_media_stream_new (object_path, c, name, id,
+      nat_traversal);
+  g_free (nat_traversal);
 
   DEBUG ("%p: created new MediaStream %p for content '%s'", chan, stream, name);
 
-  /* we will own the only reference to this stream */
   g_ptr_array_add (priv->streams, stream);
 
-  g_signal_connect (stream, "close",
-                    (GCallback) stream_close_cb, chan);
-  g_signal_connect (stream, "error",
-                    (GCallback) stream_error_cb, chan);
-  g_signal_connect (stream, "unhold-failed",
-                    (GCallback) stream_unhold_failed, chan);
-  g_signal_connect (stream, "notify::connection-state",
-                    (GCallback) stream_state_changed_cb, chan);
-  g_signal_connect (stream, "notify::combined-direction",
-                    (GCallback) stream_direction_changed_cb, chan);
-  g_signal_connect (stream, "notify::local-hold",
-                    (GCallback) stream_hold_state_changed, chan);
+  gabble_signal_connect_weak (stream, "close", (GCallback) stream_close_cb,
+      chan_o);
+  gabble_signal_connect_weak (stream, "error", (GCallback) stream_error_cb,
+      chan_o);
+  gabble_signal_connect_weak (stream, "unhold-failed",
+      (GCallback) stream_unhold_failed, chan_o);
+  gabble_signal_connect_weak (stream, "notify::connection-state",
+      (GCallback) stream_state_changed_cb, chan_o);
+  gabble_signal_connect_weak (stream, "notify::combined-direction",
+      (GCallback) stream_direction_changed_cb, chan_o);
+  gabble_signal_connect_weak (stream, "notify::local-hold",
+      (GCallback) stream_hold_state_changed, chan_o);
 
   /* emit StreamAdded */
+  mtype = gabble_media_stream_get_media_type (stream);
+
   DEBUG ("emitting StreamAdded with type '%s'",
-    type == JINGLE_MEDIA_TYPE_AUDIO ? "audio" : "video");
+    mtype == TP_MEDIA_STREAM_TYPE_AUDIO ? "audio" : "video");
 
   tp_svc_channel_type_streamed_media_emit_stream_added (
       chan, id, priv->session->peer, mtype);
@@ -2348,6 +2394,10 @@ create_stream_from_content (GabbleMediaChannel *chan, GabbleJingleContent *c)
   /* A stream being added might cause the "total" hold state to change */
   stream_hold_state_changed (stream, NULL, chan);
 
+  /* Initial stream direction was changed before we had time to hook up
+   * signal handler, so we call the handler manually to pick it up. */
+  stream_direction_changed_cb (stream, NULL, chan);
+
   if (priv->ready)
     {
       /* all of the streams are bidirectional from farsight's point of view, it's
@@ -2358,8 +2408,6 @@ create_stream_from_content (GabbleMediaChannel *chan, GabbleJingleContent *c)
     }
 
   g_free (object_path);
-
-  return stream;
 }
 
 static void
