@@ -92,6 +92,12 @@ void _gabble_media_stream_close (GabbleMediaStream *close);
 void gabble_media_stream_hold (GabbleMediaStream *stream, gboolean hold);
 gboolean gabble_media_stream_change_direction (GabbleMediaStream *stream,
     guint requested_dir, GError **error);
+void gabble_media_stream_accept_pending_local_send (GabbleMediaStream *stream);
+
+GabbleMediaStream *gabble_media_stream_new (const gchar *object_path,
+    GabbleJingleContent *content, const gchar *name, guint id,
+    const gchar *nat_traversal);
+TpMediaStreamType gabble_media_stream_get_media_type (GabbleMediaStream *self);
 
 G_END_DECLS
 
