@@ -45,6 +45,8 @@
 #include "namespaces.h"
 #include "presence-cache.h"
 #include "tubes-channel.h"
+#include "tube-dbus.h"
+#include "tube-stream.h"
 #include "util.h"
 
 static GabbleTubesChannel *new_tubes_channel (GabblePrivateTubesFactory *fac,
@@ -82,7 +84,7 @@ struct _GabblePrivateTubesFactoryPrivate
   gulong status_changed_id;
   LmMessageHandler *msg_tube_cb;
 
-  GHashTable *channels;
+  GHashTable *tubes_channels;
 
   gboolean dispose_has_run;
 };
@@ -116,8 +118,21 @@ struct _TubesCapabilities
    * gchar *ServiceName -> Feature *feature
    */
   GHashTable *dbus_tube_caps;
+
+  gboolean tubes_supported;
 };
 
+static const gchar * const tubes_channel_fixed_properties[] = {
+    TP_IFACE_CHANNEL ".ChannelType",
+    TP_IFACE_CHANNEL ".TargetHandleType",
+    NULL
+};
+
+static const gchar * const old_tubes_channel_allowed_properties[] = {
+    TP_IFACE_CHANNEL ".TargetHandle",
+    TP_IFACE_CHANNEL ".TargetID",
+    NULL
+};
 
 static void
 gabble_private_tubes_factory_init (GabblePrivateTubesFactory *self)
@@ -128,7 +143,7 @@ gabble_private_tubes_factory_init (GabblePrivateTubesFactory *self)
   self->priv = priv;
 
   priv->msg_tube_cb = NULL;
-  priv->channels = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+  priv->tubes_channels = g_hash_table_new_full (g_direct_hash, g_direct_equal,
       NULL, g_object_unref);
 
   priv->conn = NULL;
@@ -196,7 +211,7 @@ gabble_private_tubes_factory_dispose (GObject *object)
   priv->dispose_has_run = TRUE;
 
   gabble_private_tubes_factory_close_all (fac);
-  g_assert (priv->channels == NULL);
+  g_assert (priv->tubes_channels == NULL);
 
   if (G_OBJECT_CLASS (gabble_private_tubes_factory_parent_class)->dispose)
     G_OBJECT_CLASS (gabble_private_tubes_factory_parent_class)->dispose (
@@ -288,7 +303,7 @@ tubes_channel_closed_cb (GabbleTubesChannel *chan,
     GABBLE_PRIVATE_TUBES_FACTORY_GET_PRIVATE (self);
   TpHandle contact_handle;
 
-  if (priv->channels == NULL)
+  if (priv->tubes_channels == NULL)
     return;
 
   g_object_get (chan, "handle", &contact_handle, NULL);
@@ -298,7 +313,7 @@ tubes_channel_closed_cb (GabbleTubesChannel *chan,
 
   DEBUG ("removing tubes channel with handle %d", contact_handle);
 
-  g_hash_table_remove (priv->channels, GUINT_TO_POINTER (contact_handle));
+  g_hash_table_remove (priv->tubes_channels, GUINT_TO_POINTER (contact_handle));
 }
 
 /**
@@ -317,6 +332,7 @@ new_tubes_channel (GabblePrivateTubesFactory *fac,
   TpBaseConnection *conn;
   GabbleTubesChannel *chan;
   char *object_path;
+  gboolean requested;
 
   g_assert (GABBLE_IS_PRIVATE_TUBES_FACTORY (fac));
   g_assert (handle != 0);
@@ -328,19 +344,22 @@ new_tubes_channel (GabblePrivateTubesFactory *fac,
   object_path = g_strdup_printf ("%s/SITubesChannel%u", conn->object_path,
       handle);
 
+  requested = (request_token != NULL);
+
   chan = g_object_new (GABBLE_TYPE_TUBES_CHANNEL,
                        "connection", priv->conn,
                        "object-path", object_path,
                        "handle", handle,
                        "handle-type", TP_HANDLE_TYPE_CONTACT,
                        "initiator-handle", initiator,
+                       "requested", requested,
                        NULL);
 
   DEBUG ("object path %s", object_path);
 
   g_signal_connect (chan, "closed", G_CALLBACK (tubes_channel_closed_cb), fac);
 
-  g_hash_table_insert (priv->channels, GUINT_TO_POINTER (handle), chan);
+  g_hash_table_insert (priv->tubes_channels, GUINT_TO_POINTER (handle), chan);
 
   g_free (object_path);
 
@@ -387,11 +406,11 @@ gabble_private_tubes_factory_close_all (GabblePrivateTubesFactory *fac)
   /* Use a temporary variable because we don't want
    * tubes_channel_closed_cb to remove the channel from the hash table a
    * second time */
-  if (priv->channels != NULL)
+  if (priv->tubes_channels != NULL)
     {
-      GHashTable *tmp = priv->channels;
+      GHashTable *tmp = priv->tubes_channels;
 
-      priv->channels = NULL;
+      priv->tubes_channels = NULL;
       g_hash_table_destroy (tmp);
     }
 }
@@ -407,10 +426,11 @@ add_service_to_array (gchar *service,
   GValue *channel_type_value;
   GValue *target_handle_type_value;
   gchar *tube_allowed_properties[] =
-      {
+    {
         TP_IFACE_CHANNEL ".TargetHandle",
+        TP_IFACE_CHANNEL ".TargetID",
         NULL
-      };
+    };
 
   g_assert (type == TP_TUBE_TYPE_STREAM || type == TP_TUBE_TYPE_DBUS);
 
@@ -458,6 +478,73 @@ add_service_to_array (gchar *service,
   g_ptr_array_add (arr, g_value_get_boxed (&monster));
 }
 
+static void
+add_generic_tube_caps (GPtrArray *arr)
+{
+  GValue monster1 = {0,}, monster2 = {0,};
+  GHashTable *fixed_properties;
+  GValue *channel_type_value;
+  GValue *target_handle_type_value;
+
+  /* StreamTube */
+  g_value_init (&monster1, TP_STRUCT_TYPE_REQUESTABLE_CHANNEL_CLASS);
+  g_value_take_boxed (&monster1,
+      dbus_g_type_specialized_construct (
+        TP_STRUCT_TYPE_REQUESTABLE_CHANNEL_CLASS));
+
+  fixed_properties = g_hash_table_new_full (g_str_hash, g_str_equal,
+      NULL, (GDestroyNotify) tp_g_value_slice_free);
+
+  channel_type_value = tp_g_value_slice_new (G_TYPE_STRING);
+  g_value_set_static_string (channel_type_value,
+      GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE);
+
+  g_hash_table_insert (fixed_properties, TP_IFACE_CHANNEL ".ChannelType",
+      channel_type_value);
+
+  target_handle_type_value = tp_g_value_slice_new (G_TYPE_UINT);
+  g_value_set_uint (target_handle_type_value, TP_HANDLE_TYPE_CONTACT);
+  g_hash_table_insert (fixed_properties,
+      TP_IFACE_CHANNEL ".TargetHandleType", target_handle_type_value);
+
+  dbus_g_type_struct_set (&monster1,
+      0, fixed_properties,
+      1, gabble_tube_stream_channel_get_allowed_properties (),
+      G_MAXUINT);
+
+  g_hash_table_destroy (fixed_properties);
+  g_ptr_array_add (arr, g_value_get_boxed (&monster1));
+
+  /* DBusTube */
+  g_value_init (&monster2, TP_STRUCT_TYPE_REQUESTABLE_CHANNEL_CLASS);
+  g_value_take_boxed (&monster2,
+      dbus_g_type_specialized_construct (
+        TP_STRUCT_TYPE_REQUESTABLE_CHANNEL_CLASS));
+
+  fixed_properties = g_hash_table_new_full (g_str_hash, g_str_equal,
+      NULL, (GDestroyNotify) tp_g_value_slice_free);
+
+  channel_type_value = tp_g_value_slice_new (G_TYPE_STRING);
+  g_value_set_static_string (channel_type_value,
+      GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE);
+
+  g_hash_table_insert (fixed_properties, TP_IFACE_CHANNEL ".ChannelType",
+      channel_type_value);
+
+  target_handle_type_value = tp_g_value_slice_new (G_TYPE_UINT);
+  g_value_set_uint (target_handle_type_value, TP_HANDLE_TYPE_CONTACT);
+  g_hash_table_insert (fixed_properties,
+      TP_IFACE_CHANNEL ".TargetHandleType", target_handle_type_value);
+
+  dbus_g_type_struct_set (&monster2,
+      0, fixed_properties,
+      1, gabble_tube_dbus_channel_get_allowed_properties (),
+      G_MAXUINT);
+
+  g_hash_table_destroy (fixed_properties);
+  g_ptr_array_add (arr, g_value_get_boxed (&monster2));
+}
+
 static void
 gabble_private_tubes_factory_get_contact_caps (
     GabbleCapsChannelManager *manager,
@@ -483,13 +570,30 @@ gabble_private_tubes_factory_get_contact_caps (
   if (presence == NULL)
     return;
 
+  if (handle == base->self_handle &&
+      (presence->per_channel_manager_caps == NULL ||
+       g_hash_table_lookup (presence->per_channel_manager_caps, manager)
+       == NULL))
+    {
+      /* No tubes capabilities have been set but we always support at
+       * least generic tubes caps */
+      add_generic_tube_caps (arr);
+      return;
+    }
+
   if (presence->per_channel_manager_caps == NULL)
     return;
 
   caps = g_hash_table_lookup (presence->per_channel_manager_caps, manager);
+
   if (caps == NULL)
     return;
 
+  if (!caps->tubes_supported)
+    return;
+
+  add_generic_tube_caps (arr);
+
   stream_tube_caps = caps->stream_tube_caps;
   dbus_tube_caps = caps->dbus_tube_caps;
 
@@ -580,6 +684,12 @@ gabble_private_tubes_factory_parse_caps (
       if (NULL == var)
         continue;
 
+      if (!g_str_has_prefix (var, NS_TUBES))
+        continue;
+
+      /* tubes generic cap or service specific */
+      caps->tubes_supported = TRUE;
+
       if (g_str_has_prefix (var, NS_TUBES "/"))
         {
           /* http://telepathy.freedesktop.org/xmpp/tubes/$type#$service */
@@ -643,6 +753,8 @@ gabble_private_tubes_factory_copy_caps (
   g_hash_table_foreach (caps_in->dbus_tube_caps, copy_caps_helper,
       caps_out->dbus_tube_caps);
 
+  caps_out->tubes_supported = caps_in->tubes_supported;
+
   *specific_caps_out = caps_out;
 }
 
@@ -664,6 +776,24 @@ gabble_private_tubes_factory_update_caps (
       caps_in->dbus_tube_caps, (GBoxedCopyFunc) g_strdup, NULL);
 }
 
+static gboolean
+hash_table_is_subset (GHashTable *superset,
+                      GHashTable *subset)
+{
+  GHashTableIter iter;
+  gpointer look_for;
+
+  g_hash_table_iter_init (&iter, subset);
+  while (g_hash_table_iter_next (&iter, &look_for, NULL))
+    {
+      if (!g_hash_table_lookup_extended (superset, look_for, NULL, NULL))
+        /* One of subset's key is not in superset */
+        return FALSE;
+    }
+
+  return TRUE;
+}
+
 static gboolean
 gabble_private_tubes_factory_caps_diff (
     GabbleCapsChannelManager *manager,
@@ -673,64 +803,33 @@ gabble_private_tubes_factory_caps_diff (
 {
   TubesCapabilities *old_caps = specific_old_caps;
   TubesCapabilities *new_caps = specific_new_caps;
-  GHashTableIter tube_caps_iter;
-  gpointer service;
 
-  if (old_caps != NULL)
-    {
-      g_hash_table_iter_init (&tube_caps_iter, old_caps->stream_tube_caps);
-      while (g_hash_table_iter_next (&tube_caps_iter, &service,
-            NULL))
-        {
-          gpointer key, value;
-          if (new_caps == NULL ||
-              !g_hash_table_lookup_extended (new_caps->stream_tube_caps,
-                  service, &key, &value))
-            {
-              return TRUE;
-            }
-        }
-      g_hash_table_iter_init (&tube_caps_iter, old_caps->dbus_tube_caps);
-      while (g_hash_table_iter_next (&tube_caps_iter, &service,
-            NULL))
-        {
-          gpointer key, value;
-          if (new_caps == NULL ||
-              !g_hash_table_lookup_extended (new_caps->dbus_tube_caps,
-                  service, &key, &value))
-            {
-              return TRUE;
-            }
-        }
-    }
+  if (old_caps == new_caps)
+    return FALSE;
+
+  if (old_caps == NULL || new_caps == NULL)
+    return TRUE;
+
+  if (old_caps->tubes_supported != new_caps->tubes_supported)
+    return TRUE;
+
+  if (g_hash_table_size (old_caps->stream_tube_caps) !=
+      g_hash_table_size (new_caps->stream_tube_caps))
+    return TRUE;
+
+  if (g_hash_table_size (old_caps->dbus_tube_caps) !=
+      g_hash_table_size (new_caps->dbus_tube_caps))
+    return TRUE;
+
+  /* Hash tables have the same size */
+  if (!hash_table_is_subset (new_caps->stream_tube_caps,
+        old_caps->stream_tube_caps))
+    return TRUE;
+
+  if (!hash_table_is_subset (new_caps->dbus_tube_caps,
+        old_caps->dbus_tube_caps))
+    return TRUE;
 
-  if (new_caps != NULL)
-    {
-      g_hash_table_iter_init (&tube_caps_iter, new_caps->stream_tube_caps);
-      while (g_hash_table_iter_next (&tube_caps_iter, &service,
-            NULL))
-        {
-          gpointer key, value;
-          if (old_caps == NULL ||
-              !g_hash_table_lookup_extended (old_caps->stream_tube_caps,
-                  service, &key, &value))
-            {
-              return TRUE;
-            }
-        }
-      g_hash_table_iter_init (&tube_caps_iter, new_caps->dbus_tube_caps);
-      while (g_hash_table_iter_next (&tube_caps_iter, &service,
-            NULL))
-        {
-          gpointer key, value;
-          if (old_caps == NULL ||
-              !g_hash_table_lookup_extended (old_caps->dbus_tube_caps,
-                  service, &key, &value))
-            {
-              return TRUE;
-            }
-        }
-    }
   return FALSE;
 }
 
@@ -777,6 +876,10 @@ gabble_private_tubes_factory_add_cap (GabbleCapsChannelManager *manager,
       caps->dbus_tube_caps = g_hash_table_new_full (g_str_hash, g_str_equal,
           g_free, gabble_private_tubes_factory_free_feat);
       g_hash_table_insert (presence->per_channel_manager_caps, manager, caps);
+
+      if (handle == base->self_handle)
+        /* We always support generic tubes caps */
+        caps->tubes_supported = TRUE;
     }
 
   if (!tp_strdiff (channel_type, GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE))
@@ -837,7 +940,7 @@ gabble_private_tubes_factory_foreach_channel (TpChannelManager *manager,
   data.user_data = user_data;
   data.foreach = foreach;
 
-  g_hash_table_foreach (priv->channels, _foreach_slave, &data);
+  g_hash_table_foreach (priv->tubes_channels, _foreach_slave, &data);
 }
 
 void
@@ -857,7 +960,7 @@ gabble_private_tubes_factory_handle_si_tube_request (
   DEBUG ("contact#%u stream %s", handle, stream_id);
   g_return_if_fail (tp_handle_is_valid (contact_repo, handle, NULL));
 
-  chan = g_hash_table_lookup (priv->channels, GUINT_TO_POINTER (handle));
+  chan = g_hash_table_lookup (priv->tubes_channels, GUINT_TO_POINTER (handle));
   if (chan == NULL)
     {
       chan = new_tubes_channel (self, handle, handle, NULL, TRUE);
@@ -886,7 +989,7 @@ gabble_private_tubes_factory_handle_si_stream_request (
   DEBUG ("contact#%u stream %s", handle, stream_id);
   g_return_if_fail (tp_handle_is_valid (contact_repo, handle, NULL));
 
-  chan = g_hash_table_lookup (priv->channels, GUINT_TO_POINTER (handle));
+  chan = g_hash_table_lookup (priv->tubes_channels, GUINT_TO_POINTER (handle));
   if (chan == NULL)
     {
       GError e = { GABBLE_XMPP_ERROR, XMPP_ERROR_BAD_REQUEST,
@@ -939,7 +1042,7 @@ private_tubes_factory_msg_tube_cb (LmMessageHandler *handler,
     }
 
   /* Tube offer */
-  chan = g_hash_table_lookup (priv->channels, GUINT_TO_POINTER (handle));
+  chan = g_hash_table_lookup (priv->tubes_channels, GUINT_TO_POINTER (handle));
 
   if (chan == NULL)
     {
@@ -973,34 +1076,6 @@ gabble_private_tubes_factory_new (GabbleConnection *conn)
       NULL);
 }
 
-
-static const gchar * const tubes_channel_fixed_properties[] = {
-    TP_IFACE_CHANNEL ".ChannelType",
-    TP_IFACE_CHANNEL ".TargetHandleType",
-    NULL
-};
-
-static const gchar * const old_tubes_channel_allowed_properties[] = {
-    TP_IFACE_CHANNEL ".TargetHandle",
-    TP_IFACE_CHANNEL ".TargetID",
-    NULL
-};
-static const gchar * const stream_tube_channel_allowed_properties[] = {
-    TP_IFACE_CHANNEL ".TargetHandle",
-    TP_IFACE_CHANNEL ".TargetID",
-    GABBLE_IFACE_CHANNEL_INTERFACE_TUBE ".Parameters",
-    GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service",
-    NULL
-};
-static const gchar * const dbus_tube_channel_allowed_properties[] = {
-    TP_IFACE_CHANNEL ".TargetHandle",
-    TP_IFACE_CHANNEL ".TargetID",
-    GABBLE_IFACE_CHANNEL_INTERFACE_TUBE ".Parameters",
-    GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName",
-    NULL
-};
-
-
 static void
 gabble_private_tubes_factory_foreach_channel_class (
     TpChannelManager *manager,
@@ -1042,14 +1117,12 @@ gabble_private_tubes_factory_foreach_channel_class (
   g_hash_table_insert (table, TP_IFACE_CHANNEL ".TargetHandleType",
       value);
 
-  func (manager, table, stream_tube_channel_allowed_properties, user_data);
+  func (manager, table, gabble_tube_stream_channel_get_allowed_properties (),
+      user_data);
 
   g_hash_table_destroy (table);
 
   /* 1-1 Channel.Type.DBusTube */
-  /* Channel.Type.DBusTube.DRAFT is not fully implemented yet, so let's
-   * disable this. FIXME: enable this when implemented. */
-#if 0
   table = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
       (GDestroyNotify) tp_g_value_slice_free);
 
@@ -1063,10 +1136,10 @@ gabble_private_tubes_factory_foreach_channel_class (
   g_hash_table_insert (table, TP_IFACE_CHANNEL ".TargetHandleType",
       value);
 
-  func (manager, table, dbus_tube_channel_allowed_properties, user_data);
+  func (manager, table, gabble_tube_dbus_channel_get_allowed_properties (),
+      user_data);
 
   g_hash_table_destroy (table);
-#endif
 }
 
 
@@ -1089,11 +1162,6 @@ gabble_private_tubes_factory_requestotron (GabblePrivateTubesFactory *self,
   channel_type = tp_asv_get_string (request_properties,
             TP_IFACE_CHANNEL ".ChannelType");
 
-  /* Channel.Type.DBusTube.DRAFT is not fully implemented yet, so let's
-   * disable this. FIXME: enable this when implemented. */
-  if (!tp_strdiff (channel_type, GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE))
-    return FALSE;
-
   if (tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TUBES) &&
       tp_strdiff (channel_type, GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE) &&
       tp_strdiff (channel_type, GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE))
@@ -1113,7 +1181,7 @@ gabble_private_tubes_factory_requestotron (GabblePrivateTubesFactory *self,
 
       if (tp_channel_manager_asv_has_unknown_properties (request_properties,
               tubes_channel_fixed_properties,
-              stream_tube_channel_allowed_properties,
+              gabble_tube_stream_channel_get_allowed_properties (),
               &error))
         goto error;
 
@@ -1131,10 +1199,11 @@ gabble_private_tubes_factory_requestotron (GabblePrivateTubesFactory *self,
   else if (! tp_strdiff (channel_type, GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE))
     {
       const gchar *service;
+      GError *err = NULL;
 
       if (tp_channel_manager_asv_has_unknown_properties (request_properties,
               tubes_channel_fixed_properties,
-              dbus_tube_channel_allowed_properties,
+              gabble_tube_dbus_channel_get_allowed_properties (),
               &error))
         goto error;
 
@@ -1148,6 +1217,15 @@ gabble_private_tubes_factory_requestotron (GabblePrivateTubesFactory *self,
               GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName");
           goto error;
         }
+
+      if (!tp_dbus_check_valid_bus_name (service, TP_DBUS_NAME_TYPE_WELL_KNOWN,
+            &err))
+        {
+          g_set_error (&error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+              "Invalid ServiceName: %s", err->message);
+          g_error_free (err);
+          goto error;
+        }
     }
 
   /* validity already checked by TpBaseConnection */
@@ -1163,7 +1241,7 @@ gabble_private_tubes_factory_requestotron (GabblePrivateTubesFactory *self,
       goto error;
     }
 
-  channel = g_hash_table_lookup (self->priv->channels,
+  channel = g_hash_table_lookup (self->priv->tubes_channels,
       GUINT_TO_POINTER (handle));
 
   if (! tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TUBES))
@@ -1190,7 +1268,7 @@ gabble_private_tubes_factory_requestotron (GabblePrivateTubesFactory *self,
     }
   else
     {
-      gboolean channel_already_existed = (channel != NULL);
+      gboolean tubes_channel_already_existed = (channel != NULL);
       GabbleTubeIface *new_channel;
 
       if (channel == NULL)
@@ -1211,7 +1289,7 @@ gabble_private_tubes_factory_requestotron (GabblePrivateTubesFactory *self,
 
           channels = g_hash_table_new_full (g_direct_hash, g_direct_equal,
               NULL, NULL);
-          if (!channel_already_existed)
+          if (!tubes_channel_already_existed)
             g_hash_table_insert (channels, channel, NULL);
 
           if (request_token != NULL)
@@ -1277,7 +1355,6 @@ gabble_private_tubes_factory_ensure_channel (TpChannelManager *manager,
       request_properties, FALSE);
 }
 
-
 static void
 channel_manager_iface_init (gpointer g_iface,
                             gpointer iface_data)
