@@ -458,7 +458,7 @@ _gabble_roster_item_get (GabbleRoster *roster,
   g_assert (GABBLE_IS_ROSTER (roster));
   g_assert (tp_handle_is_valid (contact_repo, handle, NULL));
 
-  item = g_hash_table_lookup (priv->items, GINT_TO_POINTER (handle));
+  item = g_hash_table_lookup (priv->items, GUINT_TO_POINTER (handle));
 
   if (NULL == item)
     {
@@ -479,7 +479,7 @@ _gabble_roster_item_get (GabbleRoster *roster,
       item->name = alias;
       item->groups = tp_handle_set_new (group_repo);
       tp_handle_ref (contact_repo, handle);
-      g_hash_table_insert (priv->items, GINT_TO_POINTER (handle), item);
+      g_hash_table_insert (priv->items, GUINT_TO_POINTER (handle), item);
     }
 
   return item;
@@ -497,7 +497,7 @@ _gabble_roster_item_remove (GabbleRoster *roster,
   g_assert (GABBLE_IS_ROSTER (roster));
   g_assert (tp_handle_is_valid (contact_repo, handle, NULL));
 
-  g_hash_table_remove (priv->items, GINT_TO_POINTER (handle));
+  g_hash_table_remove (priv->items, GUINT_TO_POINTER (handle));
   tp_handle_unref (contact_repo, handle);
 }
 
@@ -915,7 +915,7 @@ _gabble_roster_create_channel (GabbleRoster *roster,
   g_assert (handle_type == TP_HANDLE_TYPE_LIST ||
             handle_type == TP_HANDLE_TYPE_GROUP);
   g_assert (channels != NULL);
-  g_assert (g_hash_table_lookup (channels, GINT_TO_POINTER (handle)) == NULL);
+  g_assert (g_hash_table_lookup (channels, GUINT_TO_POINTER (handle)) == NULL);
 
   name = tp_handle_inspect (handle_repo, handle);
   DEBUG ("Instantiating channel %u:%u \"%s\"", handle_type, handle, name);
@@ -940,7 +940,7 @@ _gabble_roster_create_channel (GabbleRoster *roster,
   g_signal_connect (chan, "closed", (GCallback) roster_channel_closed_cb,
       roster);
 
-  g_hash_table_insert (channels, GINT_TO_POINTER (handle), chan);
+  g_hash_table_insert (channels, GUINT_TO_POINTER (handle), chan);
 
   if (priv->roster_received)
     {
@@ -989,7 +989,7 @@ _gabble_roster_get_channel (GabbleRoster *roster,
 
   DEBUG ("Looking up channel %u:%u \"%s\"", handle_type, handle,
          tp_handle_inspect (handle_repo, handle));
-  chan = g_hash_table_lookup (channels, GINT_TO_POINTER (handle));
+  chan = g_hash_table_lookup (channels, GUINT_TO_POINTER (handle));
 
   if (chan == NULL)
     {
@@ -1025,7 +1025,7 @@ _gabble_roster_emit_one (gpointer key,
   GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
   TpHandleRepoIface *handle_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, data_struct->handle_type);
-  TpHandle handle = GPOINTER_TO_INT (key);
+  TpHandle handle = GPOINTER_TO_UINT (key);
   const gchar *name;
 
   g_assert (data_struct->handle_type == TP_HANDLE_TYPE_GROUP ||
@@ -1899,6 +1899,8 @@ roster_item_apply_edits (GabbleRoster *roster,
   TpHandleRepoIface *group_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_GROUP);
   GabbleRosterItemEdit *edits = item->unsent_edits;
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   LmMessage *message;
 
   DEBUG ("Applying edits to contact#%u", contact);
@@ -2016,6 +2018,9 @@ roster_item_apply_edits (GabbleRoster *roster,
     }
 
   DEBUG ("Contact#%u did change, sending message", contact);
+
+  /* keep the handle valid until roster_edited_cb runs; it will do the unref */
+  tp_handle_ref (contact_repo, contact);
   message = _gabble_roster_item_to_message (roster, contact, NULL,
       &edited_item);
   ret = _gabble_connection_send_with_reply (priv->conn,
@@ -2051,6 +2056,8 @@ roster_edited_cb (GabbleConnection *conn,
   GabbleRoster *roster = GABBLE_ROSTER (roster_obj);
   TpHandle contact = GPOINTER_TO_UINT (user_data);
   GabbleRosterItem *item = _gabble_roster_item_get (roster, contact);
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *) conn, TP_HANDLE_TYPE_CONTACT);
 
   if (item->unsent_edits)
     {
@@ -2058,6 +2065,8 @@ roster_edited_cb (GabbleConnection *conn,
       roster_item_apply_edits (roster, contact, item);
     }
 
+  tp_handle_unref (contact_repo, contact);
+
   return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 }
 
@@ -2076,7 +2085,7 @@ gabble_roster_handle_get_subscription (GabbleRoster *roster,
   g_return_val_if_fail (tp_handle_is_valid (contact_repo, handle, NULL),
       GABBLE_ROSTER_SUBSCRIPTION_NONE);
 
-  item = g_hash_table_lookup (priv->items, GINT_TO_POINTER (handle));
+  item = g_hash_table_lookup (priv->items, GUINT_TO_POINTER (handle));
 
   if (NULL == item)
     return GABBLE_ROSTER_SUBSCRIPTION_NONE;
@@ -2141,6 +2150,8 @@ gabble_roster_handle_set_blocked (GabbleRoster *roster,
   message = _gabble_roster_item_to_message (roster, handle, NULL, NULL);
   item->google_type = orig_type;
 
+  /* keep the handle valid until roster_edited_cb runs; it will do the unref */
+  tp_handle_ref (contact_repo, handle);
   ret = _gabble_connection_send_with_reply (priv->conn,
       message, roster_edited_cb, G_OBJECT (roster),
       GUINT_TO_POINTER(handle), error);
@@ -2167,7 +2178,7 @@ gabble_roster_handle_has_entry (GabbleRoster *roster,
   g_return_val_if_fail (tp_handle_is_valid (contact_repo, handle, NULL),
       FALSE);
 
-  item = g_hash_table_lookup (priv->items, GINT_TO_POINTER (handle));
+  item = g_hash_table_lookup (priv->items, GUINT_TO_POINTER (handle));
 
   return (NULL != item);
 }
@@ -2186,7 +2197,7 @@ gabble_roster_handle_get_name (GabbleRoster *roster,
   g_return_val_if_fail (tp_handle_is_valid (contact_repo, handle, NULL),
       NULL);
 
-  item = g_hash_table_lookup (priv->items, GINT_TO_POINTER (handle));
+  item = g_hash_table_lookup (priv->items, GUINT_TO_POINTER (handle));
 
   if (NULL == item)
     return NULL;
@@ -2239,6 +2250,8 @@ gabble_roster_handle_set_name (GabbleRoster *roster,
 
   lm_message_node_set_attribute (item_node, "name", name);
 
+  /* keep the handle valid until roster_edited_cb runs; it will do the unref */
+  tp_handle_ref (contact_repo, handle);
   ret = _gabble_connection_send_with_reply (priv->conn,
       message, roster_edited_cb, G_OBJECT (roster),
       GUINT_TO_POINTER(handle), error);
@@ -2288,6 +2301,8 @@ gabble_roster_handle_remove (GabbleRoster *roster,
   subscription = item->subscription;
   item->subscription = GABBLE_ROSTER_SUBSCRIPTION_REMOVE;
 
+  /* keep the handle valid until roster_edited_cb runs; it will do the unref */
+  tp_handle_ref (contact_repo, handle);
   message = _gabble_roster_item_to_message (roster, handle, NULL, NULL);
   ret = _gabble_connection_send_with_reply (priv->conn,
       message, roster_edited_cb, G_OBJECT (roster),
@@ -2347,6 +2362,8 @@ gabble_roster_handle_add (GabbleRoster *roster,
       item->unsent_edits = item_edit_new ();
     }
 
+  /* keep the handle valid until roster_edited_cb runs; it will do the unref */
+  tp_handle_ref (contact_repo, handle);
   message = _gabble_roster_item_to_message (roster, handle, NULL, NULL);
   ret = _gabble_connection_send_with_reply (priv->conn,
       message, roster_edited_cb, G_OBJECT (roster),
@@ -2408,6 +2425,8 @@ gabble_roster_handle_add_to_group (GabbleRoster *roster,
   NODE_DEBUG (message->node, "Roster item as message");
   tp_handle_set_remove (item->groups, group);
 
+  /* keep the handle valid until roster_edited_cb runs; it will do the unref */
+  tp_handle_ref (contact_repo, handle);
   ret = _gabble_connection_send_with_reply (priv->conn,
       message, roster_edited_cb, G_OBJECT (roster),
       GUINT_TO_POINTER(handle), error);
@@ -2475,6 +2494,8 @@ gabble_roster_handle_remove_from_group (GabbleRoster *roster,
     tp_handle_set_add (item->groups, group);
   tp_handle_unref (group_repo, group);
 
+  /* keep the handle valid until roster_edited_cb runs; it will do the unref */
+  tp_handle_ref (contact_repo, handle);
   ret = _gabble_connection_send_with_reply (priv->conn,
       message, roster_edited_cb, G_OBJECT (roster),
       GUINT_TO_POINTER(handle), error);
