@@ -109,7 +109,7 @@ int main (int argc, char **argv)
   g_assert (FALSE == gabble_presence_update (presence, "bar",
     GABBLE_PRESENCE_AVAILABLE, "dingoes", -1));
   gabble_presence_set_capabilities (presence, "bar",
-      PRESENCE_CAP_GOOGLE_VOICE, 0);
+      PRESENCE_CAP_GOOGLE_VOICE, NULL, 0);
 
   dump = gabble_presence_dump (presence);
   g_assert (0 == strcmp (dump,
@@ -138,7 +138,7 @@ int main (int argc, char **argv)
 
   /* give voice cap to first resource */
   gabble_presence_set_capabilities (presence, "foo",
-      PRESENCE_CAP_GOOGLE_VOICE, 0);
+      PRESENCE_CAP_GOOGLE_VOICE, NULL, 0);
 
   /* resource has voice cap */
   resource = gabble_presence_pick_resource_by_caps (presence,
