@@ -8,6 +8,7 @@ import os
 import sha
 import sys
 import time
+import random
 
 import ns
 import servicetest
@@ -69,9 +70,12 @@ def sync_stream(q, stream):
     """Used to ensure that Gabble has processed all stanzas sent to it."""
 
     iq = IQ(stream, "get")
+    id = iq['id']
     iq.addElement(('http://jabber.org/protocol/disco#info', 'query'))
     stream.send(iq)
-    q.expect('stream-iq', query_ns='http://jabber.org/protocol/disco#info')
+    q.expect('stream-iq', query_ns='http://jabber.org/protocol/disco#info',
+        predicate=(lambda event:
+            event.stanza['id'] == id and event.iq_type == 'result'))
 
 class JabberAuthenticator(xmlstream.Authenticator):
     "Trivial XML stream authenticator that accepts one username/digest pair."
@@ -99,7 +103,7 @@ class JabberAuthenticator(xmlstream.Authenticator):
 
     def streamStarted(self, root=None):
         if root:
-            self.xmlstream.sid = root.getAttribute('id')
+            self.xmlstream.sid = '%x' % random.randint(1, sys.maxint)
 
         self.xmlstream.sendHeader()
         self.xmlstream.addOnetimeObserver(
@@ -256,6 +260,24 @@ class JabberXmlStream(BaseXmlStream):
 class XmppXmlStream(BaseXmlStream):
     version = (1, 0)
 
+class GoogleXmlStream(BaseXmlStream):
+    # ???
+    version = (0, 9)
+
+    def _cb_disco_iq(self, iq):
+        if iq.getAttribute('to') == 'localhost':
+            nodes = xpath.queryForNodes(
+                "/iq/query[@xmlns='http://jabber.org/protocol/disco#info']",
+                iq)
+            query = nodes[0]
+            feature = query.addElement('feature')
+            feature['var'] = ns.GOOGLE_ROSTER
+            feature = query.addElement('feature')
+            feature['var'] = ns.GOOGLE_JINGLE_INFO
+
+            iq['type'] = 'result'
+            self.send(iq)
+
 def make_connection(bus, event_func, params=None):
     default_params = {
         'account': 'test@localhost/Resource',
@@ -379,11 +401,6 @@ def exec_test_deferred (funs, params, protocol=None, timeout=None):
 
         conn.Disconnect()
 
-        if 'GABBLE_TEST_REFDBG' in os.environ:
-            # we have to wait that Gabble timeouts so the process is properly
-            # exited and refdbg can generates its report
-            time.sleep(5.5)
-
     except dbus.DBusException, e:
         pass
 
