@@ -90,7 +90,7 @@ def test(q, bus, conn, stream):
     # Exercise basic Channel Properties from spec 0.17.7
     channel_props = group_iface.GetAll(
             'org.freedesktop.Telepathy.Channel',
-            dbus_interface='org.freedesktop.DBus.Properties')
+            dbus_interface=dbus.PROPERTIES_IFACE)
     assert channel_props.get('TargetHandle') == 0, \
             channel_props.get('TargetHandle')
     assert channel_props.get('TargetHandleType') == 0,\
@@ -118,7 +118,7 @@ def test(q, bus, conn, stream):
     # Exercise Group Properties from spec 0.17.6 (in a basic way)
     group_props = group_iface.GetAll(
             'org.freedesktop.Telepathy.Channel.Interface.Group',
-            dbus_interface='org.freedesktop.DBus.Properties')
+            dbus_interface=dbus.PROPERTIES_IFACE)
     assert 'HandleOwners' in group_props, group_props
     assert 'Members' in group_props, group_props
     assert 'LocalPendingMembers' in group_props, group_props
@@ -142,6 +142,12 @@ def test(q, bus, conn, stream):
     stream_handler.Ready(jt.get_audio_codecs_dbus())
     stream_handler.StreamState(2)
 
+    sh_props = stream_handler.GetAll(
+            'org.freedesktop.Telepathy.Media.StreamHandler',
+            dbus_interface=dbus.PROPERTIES_IFACE)
+    assert sh_props['NATTraversal'] == 'gtalk-p2p'
+    assert sh_props['CreatedLocally'] == True
+
     e = q.expect('stream-iq')
     assert e.query.name == 'jingle'
     assert e.query['action'] == 'session-initiate'
@@ -151,10 +157,17 @@ def test(q, bus, conn, stream):
 
     q.expect('stream-iq', iq_type='result')
 
+    # Call accepted
+    q.expect('dbus-signal', signal='MembersChanged')
+
     # Time passes ... afterwards we close the chan
 
     group_iface.RemoveMembers([dbus.UInt32(1)], 'closed')
 
+    # Check that we're the actor
+    e = q.expect('dbus-signal', signal='MembersChanged')
+    assert e.args[5] == self_handle
+
     # Test completed, close the connection
 
     e = q.expect('dbus-signal', signal='Close') #XXX - match against the path
