@@ -38,67 +38,57 @@ from twisted.words.xish import domish, xpath
 
 from servicetest import EventPattern
 from gabbletest import exec_test, make_result_iq, sync_stream
+from constants import *
 
-text_iface = 'org.freedesktop.Telepathy.Channel.Type.Text'
-caps_iface = 'org.freedesktop.Telepathy.' + \
-             'Connection.Interface.ContactCapabilities.DRAFT'
-contacts_iface = 'org.freedesktop.Telepathy.Connection.Interface.Contacts'
-
-ns_tubes = 'http://telepathy.freedesktop.org/xmpp/tubes'
+from caps_helper import compute_caps_hash
+from config import PACKAGE_STRING
+import ns
 
 text_fixed_properties = dbus.Dictionary({
-    'org.freedesktop.Telepathy.Channel.TargetHandleType': 1L,
-    'org.freedesktop.Telepathy.Channel.ChannelType':
-        'org.freedesktop.Telepathy.Channel.Type.Text'
+    TARGET_HANDLE_TYPE: HT_CONTACT,
+    CHANNEL_TYPE: CHANNEL_TYPE_TEXT
     })
-text_allowed_properties = dbus.Array([
-    'org.freedesktop.Telepathy.Channel.TargetHandle',
-    ])
+text_allowed_properties = dbus.Array([TARGET_HANDLE])
+
+stream_tube_fixed_properties = dbus.Dictionary({
+    TARGET_HANDLE_TYPE: HT_CONTACT,
+    CHANNEL_TYPE: CHANNEL_TYPE_STREAM_TUBE
+    })
+stream_tube_allowed_properties = dbus.Array([TARGET_HANDLE,
+    TARGET_ID, STREAM_TUBE_SERVICE])
+
+dbus_tube_fixed_properties = dbus.Dictionary({
+    TARGET_HANDLE_TYPE: HT_CONTACT,
+    CHANNEL_TYPE: CHANNEL_TYPE_DBUS_TUBE
+    })
+dbus_tube_allowed_properties = dbus.Array([TARGET_HANDLE,
+    TARGET_ID, DBUS_TUBE_SERVICE_NAME])
+
+specialized_tube_allowed_properties = dbus.Array([TARGET_HANDLE,
+    TARGET_ID])
 
 daap_fixed_properties = dbus.Dictionary({
-    'org.freedesktop.Telepathy.Channel.TargetHandleType': 1L,
-    'org.freedesktop.Telepathy.Channel.ChannelType':
-        'org.freedesktop.Telepathy.Channel.Type.StreamTube.DRAFT',
-    'org.freedesktop.Telepathy.Channel.Type.StreamTube.DRAFT.Service':
-        'daap'
+    TARGET_HANDLE_TYPE: HT_CONTACT,
+    CHANNEL_TYPE: CHANNEL_TYPE_STREAM_TUBE,
+    STREAM_TUBE_SERVICE: 'daap'
     })
-daap_allowed_properties = dbus.Array([
-    'org.freedesktop.Telepathy.Channel.TargetHandle',
-    ])
-
 http_fixed_properties = dbus.Dictionary({
-    'org.freedesktop.Telepathy.Channel.TargetHandleType': 1L,
-    'org.freedesktop.Telepathy.Channel.ChannelType':
-        'org.freedesktop.Telepathy.Channel.Type.StreamTube.DRAFT',
-    'org.freedesktop.Telepathy.Channel.Type.StreamTube.DRAFT.Service':
-        'http'
+    TARGET_HANDLE_TYPE: HT_CONTACT,
+    CHANNEL_TYPE: CHANNEL_TYPE_STREAM_TUBE,
+    STREAM_TUBE_SERVICE: 'http'
     })
-http_allowed_properties = dbus.Array([
-    'org.freedesktop.Telepathy.Channel.TargetHandle',
-    ])
 
 xiangqi_fixed_properties = dbus.Dictionary({
-    'org.freedesktop.Telepathy.Channel.TargetHandleType': 1L,
-    'org.freedesktop.Telepathy.Channel.ChannelType':
-        'org.freedesktop.Telepathy.Channel.Type.DBusTube.DRAFT',
-    'org.freedesktop.Telepathy.Channel.Type.DBusTube.DRAFT.ServiceName':
-        'com.example.Xiangqi'
+    TARGET_HANDLE_TYPE: HT_CONTACT,
+    CHANNEL_TYPE: CHANNEL_TYPE_DBUS_TUBE,
+    DBUS_TUBE_SERVICE_NAME: 'com.example.Xiangqi'
     })
-xiangqi_allowed_properties = dbus.Array([
-    'org.freedesktop.Telepathy.Channel.TargetHandle',
-    ])
 
 go_fixed_properties = dbus.Dictionary({
-    'org.freedesktop.Telepathy.Channel.TargetHandleType': 1L,
-    'org.freedesktop.Telepathy.Channel.ChannelType':
-        'org.freedesktop.Telepathy.Channel.Type.DBusTube.DRAFT',
-    'org.freedesktop.Telepathy.Channel.Type.DBusTube.DRAFT.ServiceName':
-        'com.example.Go'
+    TARGET_HANDLE_TYPE: HT_CONTACT,
+    CHANNEL_TYPE: CHANNEL_TYPE_DBUS_TUBE,
+    DBUS_TUBE_SERVICE_NAME: 'com.example.Go'
     })
-go_allowed_properties = dbus.Array([
-    'org.freedesktop.Telepathy.Channel.TargetHandle',
-    ])
-
 
 def make_presence(from_jid, type, status):
     presence = domish.Element((None, 'presence'))
@@ -115,7 +105,7 @@ def make_presence(from_jid, type, status):
     return presence
 
 def presence_add_caps(presence, ver, client, hash=None):
-    c = presence.addElement(('http://jabber.org/protocol/caps', 'c'))
+    c = presence.addElement((ns.CAPS, 'c'))
     c['node'] = client
     c['ver'] = ver
     if hash is not None:
@@ -145,17 +135,23 @@ def receive_presence_and_ask_caps(q, stream):
     id='disco1'
     to='gabble@jabber.org/resource' 
     type='get'>
-  <query xmlns='http://jabber.org/protocol/disco#info'
+  <query xmlns='""" + ns.DISCO_INFO + """'
          node='""" + node + '#' + ver + """'/>
 </iq>
 """
     stream.send(request)
 
     # receive caps
-    event = q.expect('stream-iq',
-        query_ns='http://jabber.org/protocol/disco#info')
+    event = q.expect('stream-iq', query_ns=ns.DISCO_INFO)
     caps_str = str(xpath.queryForNodes('/iq/query/feature', event.stanza))
 
+    features = []
+    for feature in xpath.queryForNodes('/iq/query/feature', event.stanza):
+        features.append(feature['var'])
+
+    # Check if the hash matches the announced capabilities
+    assert ver == compute_caps_hash(['client/pc//%s' % PACKAGE_STRING], features, [])
+
     return (event, caps_str, signaled_caps)
 
 def caps_contain(event, cap):
@@ -173,20 +169,19 @@ def caps_contain(event, cap):
 
 def test_tube_caps_from_contact(q, bus, conn, stream, contact, contact_handle, client):
 
-    conn_caps_iface = dbus.Interface(conn, caps_iface)
-    conn_contacts_iface = dbus.Interface(conn, contacts_iface)
+    conn_caps_iface = dbus.Interface(conn, CONN_IFACE_CONTACT_CAPA)
+    conn_contacts_iface = dbus.Interface(conn, CONN_IFACE_CONTACTS)
 
     # send presence with no tube cap
     presence = make_presence(contact, None, 'hello')
-    c = presence.addElement(('http://jabber.org/protocol/caps', 'c'))
+    c = presence.addElement((ns.CAPS, 'c'))
     c['node'] = client
-    c['ver'] = 'JpaYgiKL0y4fUOCTwN3WLGpaftM='
+    c['ver'] = compute_caps_hash([], [], [])
     c['hash'] = 'sha-1'
     stream.send(presence)
 
     # Gabble looks up our capabilities
-    event = q.expect('stream-iq', to=contact,
-        query_ns='http://jabber.org/protocol/disco#info')
+    event = q.expect('stream-iq', to=contact, query_ns=ns.DISCO_INFO)
     query_node = xpath.queryForNodes('/iq/query', event.stanza)[0]
     assert query_node.attributes['node'] == \
         client + '#' + c['ver']
@@ -195,13 +190,6 @@ def test_tube_caps_from_contact(q, bus, conn, stream, contact, contact_handle, c
     result = make_result_iq(stream, event.stanza)
     query = result.firstChildElement()
     query['node'] = client + '#' + c['ver']
-
-    feature = query.addElement('feature')
-    feature['var'] = 'http://jabber.org/protocol/jingle'
-    feature = query.addElement('feature')
-    feature['var'] = 'http://jabber.org/protocol/jingle/description/audio'
-    feature = query.addElement('feature')
-    feature['var'] = 'http://www.google.com/transport/p2p'
     stream.send(result)
 
     # no change in ContactCapabilities, so no signal ContactCapabilitiesChanged
@@ -217,22 +205,21 @@ def test_tube_caps_from_contact(q, bus, conn, stream, contact, contact_handle, c
     assert caps == basic_caps, caps
     # check the Contacts interface give the same caps
     caps_via_contacts_iface = conn_contacts_iface.GetContactAttributes(
-            [contact_handle], [caps_iface], False) \
-            [contact_handle][caps_iface + '/caps']
+            [contact_handle], [CONN_IFACE_CONTACT_CAPA], False) \
+            [contact_handle][CONN_IFACE_CONTACT_CAPA + '/caps']
     assert caps_via_contacts_iface == caps[contact_handle], \
                                     caps_via_contacts_iface
 
-    # send presence with 1 stream tube cap
+    # send presence with generic tubes caps
     presence = make_presence(contact, None, 'hello')
-    c = presence.addElement(('http://jabber.org/protocol/caps', 'c'))
+    c = presence.addElement((ns.CAPS, 'c'))
     c['node'] = client
-    c['ver'] = 'f5oUAlH0fcR8btEo5K0P135QReo='
+    c['ver'] = compute_caps_hash([], [ns.TUBES], [])
     c['hash'] = 'sha-1'
     stream.send(presence)
 
     # Gabble looks up our capabilities
-    event = q.expect('stream-iq', to=contact,
-        query_ns='http://jabber.org/protocol/disco#info')
+    event = q.expect('stream-iq', to=contact, query_ns=ns.DISCO_INFO)
     query_node = xpath.queryForNodes('/iq/query', event.stanza)[0]
     assert query_node.attributes['node'] == \
         client + '#' + c['ver']
@@ -242,13 +229,59 @@ def test_tube_caps_from_contact(q, bus, conn, stream, contact, contact_handle, c
     query = result.firstChildElement()
     query['node'] = client + '#' + c['ver']
     feature = query.addElement('feature')
-    feature['var'] = ns_tubes + '/stream#daap'
+    feature['var'] = ns.TUBES
+    stream.send(result)
+
+    # generic tubes capabilities
+    generic_tubes_caps = dbus.Dictionary({contact_handle:
+            [(text_fixed_properties, text_allowed_properties),
+             (stream_tube_fixed_properties, stream_tube_allowed_properties),
+             (dbus_tube_fixed_properties, dbus_tube_allowed_properties)]})
+    event = q.expect('dbus-signal', signal='ContactCapabilitiesChanged')
+    assert len(event.args) == 1
+    assert event.args[0] == generic_tubes_caps
+
+    caps = conn_caps_iface.GetContactCapabilities([contact_handle])
+    assert caps == generic_tubes_caps, caps
+    # test again, to check GetContactCapabilities does not have side effect
+    caps = conn_caps_iface.GetContactCapabilities([contact_handle])
+    assert caps == generic_tubes_caps, caps
+    # check the Contacts interface give the same caps
+    caps_via_contacts_iface = conn_contacts_iface.GetContactAttributes(
+            [contact_handle], [CONN_IFACE_CONTACT_CAPA], False) \
+            [contact_handle][CONN_IFACE_CONTACT_CAPA + '/caps']
+    assert caps_via_contacts_iface == caps[contact_handle], \
+                                    caps_via_contacts_iface
+
+
+    # send presence with 1 stream tube cap
+    presence = make_presence(contact, None, 'hello')
+    c = presence.addElement((ns.CAPS, 'c'))
+    c['node'] = client
+    c['ver'] = compute_caps_hash([], [ns.TUBES + '/stream#daap'], [])
+    c['hash'] = 'sha-1'
+    stream.send(presence)
+
+    # Gabble looks up our capabilities
+    event = q.expect('stream-iq', to=contact, query_ns=ns.DISCO_INFO)
+    query_node = xpath.queryForNodes('/iq/query', event.stanza)[0]
+    assert query_node.attributes['node'] == \
+        client + '#' + c['ver']
+
+    # send good reply
+    result = make_result_iq(stream, event.stanza)
+    query = result.firstChildElement()
+    query['node'] = client + '#' + c['ver']
+    feature = query.addElement('feature')
+    feature['var'] = ns.TUBES + '/stream#daap'
     stream.send(result)
 
     # daap capabilities
     daap_caps = dbus.Dictionary({contact_handle:
             [(text_fixed_properties, text_allowed_properties),
-             (daap_fixed_properties, daap_allowed_properties)]})
+             (stream_tube_fixed_properties, stream_tube_allowed_properties),
+             (dbus_tube_fixed_properties, dbus_tube_allowed_properties),
+             (daap_fixed_properties, specialized_tube_allowed_properties)]})
     event = q.expect('dbus-signal', signal='ContactCapabilitiesChanged')
     assert len(event.args) == 1
     assert event.args[0] == daap_caps
@@ -260,22 +293,21 @@ def test_tube_caps_from_contact(q, bus, conn, stream, contact, contact_handle, c
     assert caps == daap_caps, caps
     # check the Contacts interface give the same caps
     caps_via_contacts_iface = conn_contacts_iface.GetContactAttributes(
-            [contact_handle], [caps_iface], False) \
-            [contact_handle][caps_iface + '/caps']
+            [contact_handle], [CONN_IFACE_CONTACT_CAPA], False) \
+            [contact_handle][CONN_IFACE_CONTACT_CAPA + '/caps']
     assert caps_via_contacts_iface == caps[contact_handle], \
                                     caps_via_contacts_iface
 
     # send presence with 1 D-Bus tube cap
     presence = make_presence(contact, None, 'hello')
-    c = presence.addElement(('http://jabber.org/protocol/caps', 'c'))
+    c = presence.addElement((ns.CAPS, 'c'))
     c['node'] = client
-    c['ver'] = '4Ps2iaOc+lsFwfbasCdsBjLOQ5s='
+    c['ver'] = compute_caps_hash([], [ns.TUBES + '/dbus#com.example.Xiangqi'], [])
     c['hash'] = 'sha-1'
     stream.send(presence)
 
     # Gabble looks up our capabilities
-    event = q.expect('stream-iq', to=contact,
-        query_ns='http://jabber.org/protocol/disco#info')
+    event = q.expect('stream-iq', to=contact, query_ns=ns.DISCO_INFO)
     query_node = xpath.queryForNodes('/iq/query', event.stanza)[0]
     assert query_node.attributes['node'] == \
         client + '#' + c['ver']
@@ -285,13 +317,15 @@ def test_tube_caps_from_contact(q, bus, conn, stream, contact, contact_handle, c
     query = result.firstChildElement()
     query['node'] = client + '#' + c['ver']
     feature = query.addElement('feature')
-    feature['var'] = ns_tubes + '/dbus#com.example.Xiangqi'
+    feature['var'] = ns.TUBES + '/dbus#com.example.Xiangqi'
     stream.send(result)
 
     # xiangqi capabilities
     xiangqi_caps = dbus.Dictionary({contact_handle:
             [(text_fixed_properties, text_allowed_properties),
-            (xiangqi_fixed_properties, xiangqi_allowed_properties)]})
+             (stream_tube_fixed_properties, stream_tube_allowed_properties),
+             (dbus_tube_fixed_properties, dbus_tube_allowed_properties),
+            (xiangqi_fixed_properties, specialized_tube_allowed_properties)]})
     event = q.expect('dbus-signal', signal='ContactCapabilitiesChanged')
     assert len(event.args) == 1
     assert event.args[0] == xiangqi_caps
@@ -303,22 +337,22 @@ def test_tube_caps_from_contact(q, bus, conn, stream, contact, contact_handle, c
     assert caps == xiangqi_caps, caps
     # check the Contacts interface give the same caps
     caps_via_contacts_iface = conn_contacts_iface.GetContactAttributes(
-            [contact_handle], [caps_iface], False) \
-            [contact_handle][caps_iface + '/caps']
+            [contact_handle], [CONN_IFACE_CONTACT_CAPA], False) \
+            [contact_handle][CONN_IFACE_CONTACT_CAPA + '/caps']
     assert caps_via_contacts_iface == caps[contact_handle], \
                                     caps_via_contacts_iface
 
     # send presence with both D-Bus and stream tube caps
     presence = make_presence(contact, None, 'hello')
-    c = presence.addElement(('http://jabber.org/protocol/caps', 'c'))
+    c = presence.addElement((ns.CAPS, 'c'))
     c['node'] = client
-    c['ver'] = 'ALCBfacl4M/FKWckV1OCHfj+lt0='
+    c['ver'] = compute_caps_hash([], [ns.TUBES + '/dbus#com.example.Xiangqi',
+        ns.TUBES + '/stream#daap'], [])
     c['hash'] = 'sha-1'
     stream.send(presence)
 
     # Gabble looks up our capabilities
-    event = q.expect('stream-iq', to=contact,
-        query_ns='http://jabber.org/protocol/disco#info')
+    event = q.expect('stream-iq', to=contact, query_ns=ns.DISCO_INFO)
     query_node = xpath.queryForNodes('/iq/query', event.stanza)[0]
     assert query_node.attributes['node'] == \
         client + '#' + c['ver']
@@ -328,16 +362,18 @@ def test_tube_caps_from_contact(q, bus, conn, stream, contact, contact_handle, c
     query = result.firstChildElement()
     query['node'] = client + '#' + c['ver']
     feature = query.addElement('feature')
-    feature['var'] = ns_tubes + '/dbus#com.example.Xiangqi'
+    feature['var'] = ns.TUBES + '/dbus#com.example.Xiangqi'
     feature = query.addElement('feature')
-    feature['var'] = ns_tubes + '/stream#daap'
+    feature['var'] = ns.TUBES + '/stream#daap'
     stream.send(result)
 
     # daap + xiangqi capabilities
     daap_xiangqi_caps = dbus.Dictionary({contact_handle:
         [(text_fixed_properties, text_allowed_properties),
-        (daap_fixed_properties, daap_allowed_properties),
-        (xiangqi_fixed_properties, xiangqi_allowed_properties)]})
+        (stream_tube_fixed_properties, stream_tube_allowed_properties),
+        (dbus_tube_fixed_properties, dbus_tube_allowed_properties),
+        (daap_fixed_properties, specialized_tube_allowed_properties),
+        (xiangqi_fixed_properties, specialized_tube_allowed_properties)]})
     event = q.expect('dbus-signal', signal='ContactCapabilitiesChanged')
     assert len(event.args) == 1
     assert event.args[0] == daap_xiangqi_caps
@@ -349,22 +385,23 @@ def test_tube_caps_from_contact(q, bus, conn, stream, contact, contact_handle, c
     assert caps == daap_xiangqi_caps, caps
     # check the Contacts interface give the same caps
     caps_via_contacts_iface = conn_contacts_iface.GetContactAttributes(
-            [contact_handle], [caps_iface], False) \
-            [contact_handle][caps_iface + '/caps']
+            [contact_handle], [CONN_IFACE_CONTACT_CAPA], False) \
+            [contact_handle][CONN_IFACE_CONTACT_CAPA + '/caps']
     assert caps_via_contacts_iface == caps[contact_handle], \
                                     caps_via_contacts_iface
 
     # send presence with 4 tube caps
     presence = make_presence(contact, None, 'hello')
-    c = presence.addElement(('http://jabber.org/protocol/caps', 'c'))
+    c = presence.addElement((ns.CAPS, 'c'))
     c['node'] = client
-    c['ver'] = 'ObSHJf9W0fUDuSjmB6gmthptw+s='
+    c['ver'] = compute_caps_hash([], [ns.TUBES + '/dbus#com.example.Xiangqi',
+        ns.TUBES + '/stream#daap',  ns.TUBES + '/dbus#com.example.Go',
+        ns.TUBES + '/stream#http' ], [])
     c['hash'] = 'sha-1'
     stream.send(presence)
 
     # Gabble looks up our capabilities
-    event = q.expect('stream-iq', to=contact,
-        query_ns='http://jabber.org/protocol/disco#info')
+    event = q.expect('stream-iq', to=contact, query_ns=ns.DISCO_INFO)
     query_node = xpath.queryForNodes('/iq/query', event.stanza)[0]
     assert query_node.attributes['node'] == \
         client + '#' + c['ver']
@@ -374,22 +411,24 @@ def test_tube_caps_from_contact(q, bus, conn, stream, contact, contact_handle, c
     query = result.firstChildElement()
     query['node'] = client + '#' + c['ver']
     feature = query.addElement('feature')
-    feature['var'] = ns_tubes + '/dbus#com.example.Xiangqi'
+    feature['var'] = ns.TUBES + '/dbus#com.example.Xiangqi'
     feature = query.addElement('feature')
-    feature['var'] = ns_tubes + '/dbus#com.example.Go'
+    feature['var'] = ns.TUBES + '/dbus#com.example.Go'
     feature = query.addElement('feature')
-    feature['var'] = ns_tubes + '/stream#daap'
+    feature['var'] = ns.TUBES + '/stream#daap'
     feature = query.addElement('feature')
-    feature['var'] = ns_tubes + '/stream#http'
+    feature['var'] = ns.TUBES + '/stream#http'
     stream.send(result)
 
     # http + daap + xiangqi + go capabilities
     all_tubes_caps = dbus.Dictionary({contact_handle:
         [(text_fixed_properties, text_allowed_properties),
-        (daap_fixed_properties, daap_allowed_properties),
-        (http_fixed_properties, http_allowed_properties),
-        (xiangqi_fixed_properties, xiangqi_allowed_properties),
-        (go_fixed_properties, go_allowed_properties)]})
+        (stream_tube_fixed_properties, stream_tube_allowed_properties),
+        (dbus_tube_fixed_properties, dbus_tube_allowed_properties),
+        (daap_fixed_properties, specialized_tube_allowed_properties),
+        (http_fixed_properties, specialized_tube_allowed_properties),
+        (xiangqi_fixed_properties, specialized_tube_allowed_properties),
+        (go_fixed_properties, specialized_tube_allowed_properties)]})
     event = q.expect('dbus-signal', signal='ContactCapabilitiesChanged')
     assert len(event.args) == 1
     assert event.args[0] == all_tubes_caps
@@ -401,16 +440,17 @@ def test_tube_caps_from_contact(q, bus, conn, stream, contact, contact_handle, c
     assert caps == all_tubes_caps, caps
     # check the Contacts interface give the same caps
     caps_via_contacts_iface = conn_contacts_iface.GetContactAttributes(
-            [contact_handle], [caps_iface], False) \
-            [contact_handle][caps_iface + '/caps']
+            [contact_handle], [CONN_IFACE_CONTACT_CAPA], False) \
+            [contact_handle][CONN_IFACE_CONTACT_CAPA + '/caps']
     assert caps_via_contacts_iface == caps[contact_handle], \
                                     caps_via_contacts_iface
 
     # send presence with both D-Bus and stream tube caps
     presence = make_presence(contact, None, 'hello')
-    c = presence.addElement(('http://jabber.org/protocol/caps', 'c'))
+    c = presence.addElement((ns.CAPS, 'c'))
     c['node'] = client
-    c['ver'] = 'ALCBfacl4M/FKWckV1OCHfj+lt0='
+    c['ver'] = compute_caps_hash([], [ns.TUBES + '/dbus#com.example.Xiangqi',
+        ns.TUBES + '/stream#daap'], [])
     c['hash'] = 'sha-1'
     stream.send(presence)
 
@@ -419,8 +459,10 @@ def test_tube_caps_from_contact(q, bus, conn, stream, contact, contact_handle, c
     # daap + xiangqi capabilities
     daap_xiangqi_caps = dbus.Dictionary({contact_handle:
         [(text_fixed_properties, text_allowed_properties),
-        (daap_fixed_properties, daap_allowed_properties),
-        (xiangqi_fixed_properties, xiangqi_allowed_properties)]})
+        (stream_tube_fixed_properties, stream_tube_allowed_properties),
+        (dbus_tube_fixed_properties, dbus_tube_allowed_properties),
+        (daap_fixed_properties, specialized_tube_allowed_properties),
+        (xiangqi_fixed_properties, specialized_tube_allowed_properties)]})
     event = q.expect('dbus-signal', signal='ContactCapabilitiesChanged')
     assert len(event.args) == 1
     assert event.args[0] == daap_xiangqi_caps
@@ -432,41 +474,51 @@ def test_tube_caps_from_contact(q, bus, conn, stream, contact, contact_handle, c
     assert caps == daap_xiangqi_caps, caps
     # check the Contacts interface give the same caps
     caps_via_contacts_iface = conn_contacts_iface.GetContactAttributes(
-            [contact_handle], [caps_iface], False) \
-            [contact_handle][caps_iface + '/caps']
+            [contact_handle], [CONN_IFACE_CONTACT_CAPA], False) \
+            [contact_handle][CONN_IFACE_CONTACT_CAPA + '/caps']
     assert caps_via_contacts_iface == caps[contact_handle], \
                                     caps_via_contacts_iface
 
 def test_tube_caps_to_contact(q, bus, conn, stream):
     basic_caps = dbus.Dictionary({1:
-        [(text_fixed_properties, text_allowed_properties)]})
+        [(text_fixed_properties, text_allowed_properties),
+         (stream_tube_fixed_properties, stream_tube_allowed_properties),
+         (dbus_tube_fixed_properties, dbus_tube_allowed_properties)]})
     daap_caps = dbus.Dictionary({1:
         [(text_fixed_properties, text_allowed_properties),
-        (daap_fixed_properties, daap_allowed_properties)]})
+        (stream_tube_fixed_properties, stream_tube_allowed_properties),
+        (dbus_tube_fixed_properties, dbus_tube_allowed_properties),
+        (daap_fixed_properties, specialized_tube_allowed_properties)]})
     xiangqi_caps = dbus.Dictionary({1:
         [(text_fixed_properties, text_allowed_properties),
-        (xiangqi_fixed_properties, xiangqi_allowed_properties)]})
+        (stream_tube_fixed_properties, stream_tube_allowed_properties),
+        (dbus_tube_fixed_properties, dbus_tube_allowed_properties),
+        (xiangqi_fixed_properties, specialized_tube_allowed_properties)]})
     daap_xiangqi_caps = dbus.Dictionary({1:
         [(text_fixed_properties, text_allowed_properties),
-        (daap_fixed_properties, daap_allowed_properties),
-        (xiangqi_fixed_properties, xiangqi_allowed_properties)]})
+        (stream_tube_fixed_properties, stream_tube_allowed_properties),
+        (dbus_tube_fixed_properties, dbus_tube_allowed_properties),
+        (daap_fixed_properties, specialized_tube_allowed_properties),
+        (xiangqi_fixed_properties, specialized_tube_allowed_properties)]})
     all_tubes_caps = dbus.Dictionary({1:
         [(text_fixed_properties, text_allowed_properties),
-        (daap_fixed_properties, daap_allowed_properties),
-        (http_fixed_properties, http_allowed_properties),
-        (xiangqi_fixed_properties, xiangqi_allowed_properties),
-        (go_fixed_properties, go_allowed_properties)]})
+        (stream_tube_fixed_properties, stream_tube_allowed_properties),
+        (dbus_tube_fixed_properties, dbus_tube_allowed_properties),
+        (daap_fixed_properties, specialized_tube_allowed_properties),
+        (http_fixed_properties, specialized_tube_allowed_properties),
+        (xiangqi_fixed_properties, specialized_tube_allowed_properties),
+        (go_fixed_properties, specialized_tube_allowed_properties)]})
 
-    conn_caps_iface = dbus.Interface(conn, caps_iface)
-    conn_contacts_iface = dbus.Interface(conn, contacts_iface)
+    conn_caps_iface = dbus.Interface(conn, CONN_IFACE_CONTACT_CAPA)
+    conn_contacts_iface = dbus.Interface(conn, CONN_IFACE_CONTACTS)
 
     # Check our own caps
     caps = conn_caps_iface.GetContactCapabilities([1])
     assert caps == basic_caps, caps
     # check the Contacts interface give the same caps
     caps_via_contacts_iface = conn_contacts_iface.GetContactAttributes(
-            [1], [caps_iface], False) \
-            [1][caps_iface + '/caps']
+            [1], [CONN_IFACE_CONTACT_CAPA], False) \
+            [1][CONN_IFACE_CONTACT_CAPA + '/caps']
     assert caps_via_contacts_iface == caps[1], caps_via_contacts_iface
 
     # Advertise nothing
@@ -478,8 +530,8 @@ def test_tube_caps_to_contact(q, bus, conn, stream):
     assert caps == basic_caps, caps
     # check the Contacts interface give the same caps
     caps_via_contacts_iface = conn_contacts_iface.GetContactAttributes(
-            [1], [caps_iface], False) \
-            [1][caps_iface + '/caps']
+            [1], [CONN_IFACE_CONTACT_CAPA], False) \
+            [1][CONN_IFACE_CONTACT_CAPA + '/caps']
     assert caps_via_contacts_iface == caps[1], caps_via_contacts_iface
 
     sync_stream(q, stream)
@@ -490,12 +542,12 @@ def test_tube_caps_to_contact(q, bus, conn, stream):
 
     # Expect Gabble to reply with the correct caps
     event, caps_str, signaled_caps = receive_presence_and_ask_caps(q, stream)
-    assert caps_contain(event, ns_tubes) == True, caps_str
-    assert caps_contain(event, ns_tubes + '/stream#daap') == True, caps_str
-    assert caps_contain(event, ns_tubes + '/stream#http') == False, caps_str
-    assert caps_contain(event, ns_tubes + '/dbus#com.example.Go') \
+    assert caps_contain(event, ns.TUBES) == True, caps_str
+    assert caps_contain(event, ns.TUBES + '/stream#daap') == True, caps_str
+    assert caps_contain(event, ns.TUBES + '/stream#http') == False, caps_str
+    assert caps_contain(event, ns.TUBES + '/dbus#com.example.Go') \
             == False, caps_str
-    assert caps_contain(event, ns_tubes + '/dbus#com.example.Xiangqi') \
+    assert caps_contain(event, ns.TUBES + '/dbus#com.example.Xiangqi') \
             == False, caps_str
     assert signaled_caps == daap_caps
 
@@ -504,8 +556,8 @@ def test_tube_caps_to_contact(q, bus, conn, stream):
     assert caps == daap_caps, caps
     # check the Contacts interface give the same caps
     caps_via_contacts_iface = conn_contacts_iface.GetContactAttributes(
-            [1], [caps_iface], False) \
-            [1][caps_iface + '/caps']
+            [1], [CONN_IFACE_CONTACT_CAPA], False) \
+            [1][CONN_IFACE_CONTACT_CAPA + '/caps']
     assert caps_via_contacts_iface == caps[1], caps_via_contacts_iface
 
     # Advertise xiangqi
@@ -514,12 +566,12 @@ def test_tube_caps_to_contact(q, bus, conn, stream):
 
     # Expect Gabble to reply with the correct caps
     event, caps_str, signaled_caps = receive_presence_and_ask_caps(q, stream)
-    assert caps_contain(event, ns_tubes) == True, caps_str
-    assert caps_contain(event, ns_tubes + '/stream#daap') == False, caps_str
-    assert caps_contain(event, ns_tubes + '/stream#http') == False, caps_str
-    assert caps_contain(event, ns_tubes + '/dbus#com.example.Go') \
+    assert caps_contain(event, ns.TUBES) == True, caps_str
+    assert caps_contain(event, ns.TUBES + '/stream#daap') == False, caps_str
+    assert caps_contain(event, ns.TUBES + '/stream#http') == False, caps_str
+    assert caps_contain(event, ns.TUBES + '/dbus#com.example.Go') \
             == False, caps_str
-    assert caps_contain(event, ns_tubes + '/dbus#com.example.Xiangqi') \
+    assert caps_contain(event, ns.TUBES + '/dbus#com.example.Xiangqi') \
             == True, caps_str
     assert signaled_caps == xiangqi_caps
 
@@ -528,8 +580,8 @@ def test_tube_caps_to_contact(q, bus, conn, stream):
     assert caps == xiangqi_caps, caps
     # check the Contacts interface give the same caps
     caps_via_contacts_iface = conn_contacts_iface.GetContactAttributes(
-            [1], [caps_iface], False) \
-            [1][caps_iface + '/caps']
+            [1], [CONN_IFACE_CONTACT_CAPA], False) \
+            [1][CONN_IFACE_CONTACT_CAPA + '/caps']
     assert caps_via_contacts_iface == caps[1], caps_via_contacts_iface
 
     # Advertise daap + xiangqi
@@ -538,12 +590,12 @@ def test_tube_caps_to_contact(q, bus, conn, stream):
 
     # Expect Gabble to reply with the correct caps
     event, caps_str, signaled_caps = receive_presence_and_ask_caps(q, stream)
-    assert caps_contain(event, ns_tubes) == True, caps_str
-    assert caps_contain(event, ns_tubes + '/stream#daap') == True, caps_str
-    assert caps_contain(event, ns_tubes + '/stream#http') == False, caps_str
-    assert caps_contain(event, ns_tubes + '/dbus#com.example.Go') \
+    assert caps_contain(event, ns.TUBES) == True, caps_str
+    assert caps_contain(event, ns.TUBES + '/stream#daap') == True, caps_str
+    assert caps_contain(event, ns.TUBES + '/stream#http') == False, caps_str
+    assert caps_contain(event, ns.TUBES + '/dbus#com.example.Go') \
             == False, caps_str
-    assert caps_contain(event, ns_tubes + '/dbus#com.example.Xiangqi') \
+    assert caps_contain(event, ns.TUBES + '/dbus#com.example.Xiangqi') \
             == True, caps_str
     assert signaled_caps == daap_xiangqi_caps
 
@@ -552,8 +604,8 @@ def test_tube_caps_to_contact(q, bus, conn, stream):
     assert caps == daap_xiangqi_caps, caps
     # check the Contacts interface give the same caps
     caps_via_contacts_iface = conn_contacts_iface.GetContactAttributes(
-            [1], [caps_iface], False) \
-            [1][caps_iface + '/caps']
+            [1], [CONN_IFACE_CONTACT_CAPA], False) \
+            [1][CONN_IFACE_CONTACT_CAPA + '/caps']
     assert caps_via_contacts_iface == caps[1], caps_via_contacts_iface
 
     # Advertise 4 tubes
@@ -563,12 +615,12 @@ def test_tube_caps_to_contact(q, bus, conn, stream):
 
     # Expect Gabble to reply with the correct caps
     event, caps_str, signaled_caps = receive_presence_and_ask_caps(q, stream)
-    assert caps_contain(event, ns_tubes) == True, caps_str
-    assert caps_contain(event, ns_tubes + '/stream#daap') == True, caps_str
-    assert caps_contain(event, ns_tubes + '/stream#http') == True, caps_str
-    assert caps_contain(event, ns_tubes + '/dbus#com.example.Go') \
+    assert caps_contain(event, ns.TUBES) == True, caps_str
+    assert caps_contain(event, ns.TUBES + '/stream#daap') == True, caps_str
+    assert caps_contain(event, ns.TUBES + '/stream#http') == True, caps_str
+    assert caps_contain(event, ns.TUBES + '/dbus#com.example.Go') \
             == True, caps_str
-    assert caps_contain(event, ns_tubes + '/dbus#com.example.Xiangqi') \
+    assert caps_contain(event, ns.TUBES + '/dbus#com.example.Xiangqi') \
             == True, caps_str
     assert signaled_caps == all_tubes_caps
 
@@ -577,8 +629,8 @@ def test_tube_caps_to_contact(q, bus, conn, stream):
     assert caps == all_tubes_caps, caps
     # check the Contacts interface give the same caps
     caps_via_contacts_iface = conn_contacts_iface.GetContactAttributes(
-            [1], [caps_iface], False) \
-            [1][caps_iface + '/caps']
+            [1], [CONN_IFACE_CONTACT_CAPA], False) \
+            [1][CONN_IFACE_CONTACT_CAPA + '/caps']
     assert caps_via_contacts_iface == caps[1], caps_via_contacts_iface
 
     # Advertise daap + xiangqi
@@ -587,12 +639,12 @@ def test_tube_caps_to_contact(q, bus, conn, stream):
 
     # Expect Gabble to reply with the correct caps
     event, caps_str, signaled_caps = receive_presence_and_ask_caps(q, stream)
-    assert caps_contain(event, ns_tubes) == True, caps_str
-    assert caps_contain(event, ns_tubes + '/stream#daap') == True, caps_str
-    assert caps_contain(event, ns_tubes + '/stream#http') == False, caps_str
-    assert caps_contain(event, ns_tubes + '/dbus#com.example.Go') \
+    assert caps_contain(event, ns.TUBES) == True, caps_str
+    assert caps_contain(event, ns.TUBES + '/stream#daap') == True, caps_str
+    assert caps_contain(event, ns.TUBES + '/stream#http') == False, caps_str
+    assert caps_contain(event, ns.TUBES + '/dbus#com.example.Go') \
             == False, caps_str
-    assert caps_contain(event, ns_tubes + '/dbus#com.example.Xiangqi') \
+    assert caps_contain(event, ns.TUBES + '/dbus#com.example.Xiangqi') \
             == True, caps_str
     assert signaled_caps == daap_xiangqi_caps
 
@@ -601,8 +653,8 @@ def test_tube_caps_to_contact(q, bus, conn, stream):
     assert caps == daap_xiangqi_caps, caps
     # check the Contacts interface give the same caps
     caps_via_contacts_iface = conn_contacts_iface.GetContactAttributes(
-            [1], [caps_iface], False) \
-            [1][caps_iface + '/caps']
+            [1], [CONN_IFACE_CONTACT_CAPA], False) \
+            [1][CONN_IFACE_CONTACT_CAPA + '/caps']
     assert caps_via_contacts_iface == caps[1], caps_via_contacts_iface
 
 
