@@ -43,7 +43,7 @@ def test(q, bus, conn, stream):
     # Exercise basic Channel Properties from spec 0.17.7
     channel_props = text_chan.GetAll(
             'org.freedesktop.Telepathy.Channel',
-            dbus_interface='org.freedesktop.DBus.Properties')
+            dbus_interface=dbus.PROPERTIES_IFACE)
     assert channel_props.get('TargetHandle') == foo_handle,\
             (channel_props.get('TargetHandle'), foo_handle)
     assert channel_props.get('TargetHandleType') == 1,\
@@ -75,7 +75,7 @@ def test(q, bus, conn, stream):
     assert body.children[0] == u'hey'
 
     # <message type="chat"><body>hello</body</message>
-    m = domish.Element(('', 'message'))
+    m = domish.Element((None, 'message'))
     m['from'] = 'foo@bar.com/Pidgin'
     m['type'] = 'chat'
     m.addElement('body', content='hello')
