@@ -4,6 +4,7 @@ import dbus
 
 from servicetest import call_async, EventPattern, tp_name_prefix
 from gabbletest import exec_test, make_result_iq, acknowledge_iq
+import ns
 
 from twisted.words.xish import domish
 
@@ -14,14 +15,6 @@ sample_parameters = dbus.Dictionary({
     'i': dbus.Int32(-123),
     }, signature='sv')
 
-NS_TUBES = 'http://telepathy.freedesktop.org/xmpp/tubes'
-NS_SI = 'http://jabber.org/protocol/si'
-NS_FEATURE_NEG = 'http://jabber.org/protocol/feature-neg'
-NS_IBB = 'http://jabber.org/protocol/ibb'
-NS_MUC_BYTESTREAM = 'http://telepathy.freedesktop.org/xmpp/protocol/muc-bytestream'
-NS_X_DATA = 'jabber:x:data'
-
-
 def test(q, bus, conn, stream):
     conn.Connect()
 
@@ -72,15 +65,19 @@ def test(q, bus, conn, stream):
     item['role'] = 'participant'
     stream.send(presence)
 
-    q.expect('dbus-signal', signal='MembersChanged',
-            args=[u'', [2, 3], [], [], [], 0, 0])
+    new_chans, members, event = q.expect_many(
+        EventPattern('dbus-signal', signal='NewChannels'),
+        EventPattern('dbus-signal', signal='MembersChanged',
+            args=[u'', [2, 3], [], [], [], 0, 0]),
+        EventPattern('dbus-return', method='RequestChannel'))
+
+    channels = new_chans.args[0]
+    assert len(channels) == 2
 
     assert conn.InspectHandles(1, [2]) == ['chat@conf.localhost/test']
     assert conn.InspectHandles(1, [3]) == ['chat@conf.localhost/bob']
     bob_handle = 3
 
-    event = q.expect('dbus-return', method='RequestChannel')
-
     tubes_chan = bus.get_object(conn.bus_name, event.value[0])
     tubes_iface_muc = dbus.Interface(tubes_chan,
             tp_name_prefix + '.Channel.Type.Tubes')
